import numpy as np

from .. import lasso, group_lasso, strong_rules, warm_start
from ..basil import basil_inner_loop, basil
from ...tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_lasso_agreement(n=200,p=50):
    '''
    check to see if it agrees with lasso path

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    groups = np.arange(p)
    group_lasso1 = group_lasso.gaussian(X, 
                                        Y, 
                                        groups)
    lagrange_sequence = group_lasso.default_lagrange_sequence(group_lasso1.penalty,
                                                              group_lasso1.grad_solution,
                                                              nstep=23) # initialized at "null" model
    sol1 = strong_rules(group_lasso1, 
                        lagrange_sequence, 
                        (group_lasso1.solution, group_lasso1.grad_solution),
                        inner_tol=1.e-12)

    weights = np.ones(p)
    lasso2 = lasso.gaussian(X, 
                            Y, 
                            weights)
    sol2 = strong_rules(lasso2, 
                        lagrange_sequence, 
                        (lasso2.solution, lasso2.grad_solution),
                        inner_tol=1.e-12)
    beta1 = sol1['beta']
    beta2 = sol2['beta']

    np.testing.assert_allclose(beta1, beta2)

@set_seed_for_test()
def test_path_subsample(n=200,p=50):
    '''
    compare a subsample path to the full path on subsampled data

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    cases = range(n//2)
    groups = ['a', 'a', 'a', 'b', 'b']
    for i in range(9):
        groups.extend([str(i)]*5)

    np.random.shuffle(groups)
    group_lasso1 = group_lasso.gaussian(X, 
                                        Y, 
                                        groups)
    group_lasso1 = group_lasso1.subsample(cases)
    lagrange_sequence = group_lasso.default_lagrange_sequence(group_lasso1.penalty,
                                                              group_lasso1.grad_solution,
                                                              nstep=23) # initialized at "null" model
    sol1 = strong_rules(group_lasso1, 
                        lagrange_sequence, 
                        (group_lasso1.solution, group_lasso1.grad_solution),
                        inner_tol=1.e-10)
    beta1 = sol1['beta']

    group_lasso2 = group_lasso.gaussian(X[cases],
                                        Y[cases],
                                        groups)
    sol2 = strong_rules(group_lasso2, 
                        lagrange_sequence, 
                        (group_lasso2.solution, group_lasso2.grad_solution),
                        inner_tol=1.e-10)
    beta2 = sol2['beta']

    np.testing.assert_allclose(beta1, beta2, rtol=1.e-3)



@set_seed_for_test()
def test_path():
    '''
    run a basic path algorithm

    '''
    n, p = 200, 50
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    np.random.shuffle(betaX)
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    groups = ['a', 'a', 'a', 'b', 'b']
    for i in range(9):
        groups.extend([str(i)]*5)

    np.random.shuffle(groups)
    group_lasso1 = group_lasso.gaussian(X, 
                                        Y, 
                                        groups)
    lagrange_sequence = group_lasso.default_lagrange_sequence(group_lasso1.penalty,
                                                              group_lasso1.grad_solution,
                                                              nstep=23) # initialized at "null" model
    sol1 = strong_rules(group_lasso1, 
                        lagrange_sequence, 
                        (group_lasso1.solution, group_lasso1.grad_solution),
                        inner_tol=1.e-5)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_unpenalized(n=200, p=50):
    '''
    run a basic path algorithm with some unpenalized variables

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    groups = ['a', 'a', 'a', 'b', 'b']
    for i in range(9):
        groups.extend([str(i)]*5)

    weights = dict([(g,1) for g in np.unique(groups)])
    weights['2'] = weights['3'] = 0
    weights['a'] = weights['b'] = 2

    group_lasso1 = group_lasso.gaussian(X, 
                                        Y, 
                                        groups,
                                        weights=weights)
    lagrange_sequence = group_lasso.default_lagrange_sequence(group_lasso1.penalty,
                                                              group_lasso1.grad_solution,
                                                              nstep=23) # initialized at "null" model
    sol1 = strong_rules(group_lasso1, 
                        lagrange_sequence, 
                        (group_lasso1.solution, group_lasso1.grad_solution), 
                        inner_tol=1.e-5)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_elastic_net(n=200, p=50):
    '''
    run a basic elastic net path algorithm

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    np.random.shuffle(betaX)
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    enet = np.zeros(X.shape[1])
    enet[4:7] = 0

    groups = ['a', 'a', 'a', 'b', 'b']
    for i in range(9):
        groups.extend([str(i)]*5)

    group_lasso1 = group_lasso.gaussian(X, 
                                        Y, 
                                        groups,
                                        alpha=0.5,
                                        elastic_net_param=enet)
    lagrange_sequence = group_lasso.default_lagrange_sequence(group_lasso1.penalty,
                                                              group_lasso1.grad_solution,
                                                              nstep=23) # initialized at "null" model
    sol1 = strong_rules(group_lasso1, 
                        lagrange_sequence, 
                        (group_lasso1.solution, group_lasso1.grad_solution),
                        inner_tol=1.e-5)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_elastic_net_unpenalized(n=200, p=50):
    '''
    run a basic elastic net path algorithm with unpenalized

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    enet = np.ones(X.shape[1])
    enet[4:8] = 0

    groups = ['a', 'a', 'a', 'b', 'b']
    for i in range(9):
        groups.extend([str(i)]*5)

    weights = dict([(g,1) for g in np.unique(groups)])
    weights['2'] = weights['3'] = 0
    weights['a'] = weights['b'] = 2

    group_lasso1 = group_lasso.gaussian(X, 
                                        Y, 
                                        groups,
                                        weights=weights,
                                        alpha=0.5,
                                        elastic_net_param=enet)
    lagrange_sequence = group_lasso.default_lagrange_sequence(group_lasso1.penalty,
                                                              group_lasso1.grad_solution,
                                                              nstep=23) # initialized at "null" model
    sol1 = strong_rules(group_lasso1, 
                        lagrange_sequence, 
                        (group_lasso1.solution, group_lasso1.grad_solution), 
                        inner_tol=1.e-5)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_basil_inner_loop(n=1000,p=600):
    '''
    test one run of the BASIL inner loop

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = np.array([3,4,5]) * np.sqrt(n)
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    enet = np.ones(X.shape[1])
    enet[4:8] = 0

    groups = ['a', 'a', 'a', 'b', 'b']
    i = 0
    while True:
        groups.extend([str(i)]*5)
        i += 1
        if len(groups) >= p:
            break
    groups = groups[:p]

    weights = dict([(g,1) for g in np.unique(groups)])
    weights['2'] = weights['3'] = 0
    weights['a'] = weights['b'] = 2

    group_lasso1 = group_lasso.gaussian(X, 
                                        Y, 
                                        groups,
                                        weights=weights,
                                        alpha=0.5,
                                        elastic_net_param=enet)
    lagrange_sequence = group_lasso.default_lagrange_sequence(group_lasso1.penalty,
                                                              group_lasso1.grad_solution,
                                                              nstep=100) # initialized at "null" model
    sol1 = basil_inner_loop(group_lasso1, 
                            lagrange_sequence[:50], 
                            (group_lasso1.solution.copy(), group_lasso1.grad_solution.copy()),
                            inner_tol=1.e-14,
                            step_nvar=10)
    lagrange1, beta1, grad1, active1 = sol1
    print(np.array(beta1).shape, 'chunk of path')

    print(active1, 'active')

@set_seed_for_test()
def test_basil(n=200,p=100):
    '''
    run BASIL 

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = np.array([3,4,5]) * np.sqrt(n)
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    groups = ['a', 'a', 'a', 'b', 'b']
    i = 0
    while True:
        groups.extend([str(i)]*5)
        i += 1
        if len(groups) >= p:
            break
    groups = groups[:p]

    weights = dict([(g,1) for g in np.unique(groups)])
    weights['a'] = weights['b'] = 2

    group_lasso1 = group_lasso.gaussian(X, 
                                        Y, 
                                        groups,
                                        weights=weights)
    print(group_lasso1.penalty)
    print(np.linalg.norm(group_lasso1.solution))
    lagrange_sequence = group_lasso.default_lagrange_sequence(group_lasso1.penalty,
                                                              group_lasso1.grad_solution,
                                                              nstep=100) # initialized at "null" model
    sol1 = basil(group_lasso1, 
                 lagrange_sequence, 
                 (group_lasso1.solution.copy(), group_lasso1.grad_solution.copy()),
                 inner_tol=1.e-14,
                 step_nvar=10,
                 step_lagrange=20)

    group_lasso2 = group_lasso.gaussian(X, 
                                        Y, 
                                        groups,
                                        weights=weights)
    print(group_lasso2.penalty)
    print(np.linalg.norm(group_lasso2.solution))
    sol2 = warm_start(group_lasso2, 
                      lagrange_sequence, 
                      (group_lasso2.solution.copy(), group_lasso2.grad_solution.copy()),
                      inner_tol=1.e-14)['beta']

    assert(np.linalg.norm(sol1 - sol2) / np.linalg.norm(sol2) < 1.e-4)

@set_seed_for_test()
def test_basil_unpenalized(n=200,p=100):
    '''
    run BASIL w/ unpenalized variables

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = np.array([3,4,5]) * np.sqrt(n)
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    enet = np.ones(X.shape[1])
    enet[4:8] = 0

    groups = ['a', 'a', 'a', 'b', 'b']
    i = 0
    while True:
        groups.extend([str(i)]*5)
        i += 1
        if len(groups) >= p:
            break
    groups = groups[:p]

    weights = dict([(g,1) for g in np.unique(groups)])
    weights['a'] = weights['b'] = 2
    weights['2'] = weights['3'] = 0
    group_lasso1 = group_lasso.gaussian(X, 
                                        Y, 
                                        groups,
                                        weights=weights,
                                        alpha=0.5,
                                        elastic_net_param=enet)

    lagrange_sequence = group_lasso.default_lagrange_sequence(group_lasso1.penalty,
                                                              group_lasso1.grad_solution,
                                                              nstep=100) # initialized at "null" model
    sol1 = basil(group_lasso1, 
                 lagrange_sequence, 
                 (group_lasso1.solution.copy(), group_lasso1.grad_solution.copy()),
                 inner_tol=1.e-14,
                 step_nvar=10,
                 step_lagrange=20)

    group_lasso2 = group_lasso.gaussian(X, 
                                        Y, 
                                        groups,
                                        weights=weights,
                                        alpha=0.5,
                                        elastic_net_param=enet)
    sol2 = warm_start(group_lasso2, 
                      lagrange_sequence, 
                      (group_lasso2.solution.copy(), group_lasso2.grad_solution.copy()),
                      inner_tol=1.e-14)['beta']

    assert(np.linalg.norm(sol1 - sol2) / np.linalg.norm(sol2) < 1.e-4)


@set_seed_for_test()
def test_basil_enet(n=200,p=100):
    '''
    run BASIL w/enet

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = np.array([3,4,5]) * np.sqrt(n)
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    enet = np.ones(X.shape[1])
    enet[4:8] = 0

    groups = ['a', 'a', 'a', 'b', 'b']
    i = 0
    while True:
        groups.extend([str(i)]*5)
        i += 1
        if len(groups) >= p:
            break
    groups = groups[:p]

    weights = dict([(g,1) for g in np.unique(groups)])
    weights['a'] = weights['b'] = 2

    group_lasso1 = group_lasso.gaussian(X, 
                                        Y, 
                                        groups,
                                        weights=weights,
                                        alpha=0.5,
                                        elastic_net_param=enet)

    lagrange_sequence = group_lasso.default_lagrange_sequence(group_lasso1.penalty,
                                                              group_lasso1.grad_solution,
                                                              nstep=100) # initialized at "null" model
    sol1 = basil(group_lasso1, 
                 lagrange_sequence, 
                 (group_lasso1.solution.copy(), group_lasso1.grad_solution.copy()),
                 inner_tol=1.e-14,
                 step_nvar=10,
                 step_lagrange=20)

    group_lasso2 = group_lasso.gaussian(X, 
                                        Y, 
                                        groups,
                                        weights=weights,
                                        alpha=0.5,
                                        elastic_net_param=enet)
    sol2 = warm_start(group_lasso2, 
                      lagrange_sequence, 
                      (group_lasso2.solution.copy(), group_lasso2.grad_solution.copy()),
                      inner_tol=1.e-14)['beta']

    rel_errors = np.array([np.linalg.norm(sol1[i]-sol2[i]) / np.linalg.norm(sol1[i])
                           for i in range(1, sol1.shape[0])])

    assert(np.percentile(rel_errors, 90) < 1.e-4)





import numpy as np
try:
    from sklearn.model_selection import cross_validate
    from sklearn.model_selection import GridSearchCV
    from ..api import (cox,
                       cox_lagrange)
    have_sklearn = True
except ImportError:
    have_sklearn = False

from ...api import l1norm
from ...tests.decorators import set_seed_for_test


@np.testing.dec.skipif(not have_sklearn)
@set_seed_for_test()
def test_cox():

    n, p = 100, 20
    X = np.random.standard_normal((n, p))
    Y = np.random.exponential(1, size=(n,))
    C = np.random.binomial(1, 0.5, size=(n,))
    response = np.array([Y, C]).T

    atom = l1norm
    atom_args = {'shape':p, 'bound':3}

    cox_lasso = cox(atom,
                    atom_args)

    cox_lasso.fit(X, response)
    print(cross_validate(cox_lasso, X, response, cv=10))

    # grid search
    params = {'atom_params':[{'shape':p,
                              'bound': b} for b in [3, 4, 5]]}
    lasso_cv = GridSearchCV(cox_lasso, params, cv=3)
    lasso_cv.fit(X, response)

    cox_lasso_offset = cox(atom,
                           atom_args,
                           offset=True)
    response_offset = np.hstack([response, np.random.standard_normal((n, 1))])
    cox_lasso_offset.fit(X, response_offset)

    cox_lasso_weights = cox(atom,
                            atom_args,
                            case_weights=True,
                            score_method='C-index')

    response_weights = np.hstack([response, np.ones((n, 1))])
    cox_lasso_weights.fit(X, response_weights)

    np.testing.assert_allclose(cox_lasso_weights._coefs,
                               cox_lasso._coefs)

    GridSearchCV(cox_lasso_offset, params, cv=3).fit(X, response_offset)
    GridSearchCV(cox_lasso_weights, params, cv=3).fit(X, response_weights)

    cox_lasso_both = cox(atom,
                         atom_args,
                         offset=True,
                         case_weights=True,
                         score_method='R2')
    response_both = np.hstack([response, np.ones((n, 1)), response_offset[:,-1:]])
    GridSearchCV(cox_lasso_both, params, cv=3).fit(X, response_both)






from __future__ import print_function, division, absolute_import

from operator import add, mul
import warnings

import numpy as np
from scipy import sparse

def broadcast_first(a, b, op):
    """ apply binary operation `op`, broadcast `a` over axis 1 if necessary

    Parameters
    ----------
    a : ndarray
        If a is 1D shape (N,), convert to shape (N,1) before appling `op`.  This
        has the effect of making broadcasting happen over axis 1 rather than the
        default of axis 0.
    b : ndarray
        If a is 1D shape (P,), convert to shape (N,1) before appling `op`
    op : callable
        binary operation to apply to `a`, `b`

    Returns
    -------
    res : object
        shape equal to ``b.shape``
    """
    shape = b.shape
    if a.ndim == 1:
        a = a[:,None]
    if b.ndim == 1:
        b = b[:,None]
    return op(a, b).reshape(shape)


class AffineError(Exception):
    pass


class affine_transform(object):
    
    def __init__(self, linear_operator, affine_offset, diag=False, input_shape=None):
        """ Create affine transform

        Parameters
        ----------
        linear_operator : None or ndarray or sparse array or affine_transform
            Linear part of affine transform implemented as array or as
            affine_transform.  None results in no linear component.
        affine_offset : None or ndarray
            offset component of affine.  Only one of `linear_operator` and
            `affine_offset` can be None, because we need an input array to
            define the shape of the transform.
        diag : {False, True}, optional
            If True, interpret 1D `linear_operator` as the main diagonal of the
            a diagonal array, so that ``linear_operator =
            np.diag(linear_operator)``
        """
        # noneD - linear_operator is None
        # sparseD - linear_operator is sparse
        # affineD - linear_operator is an affine_transform
        # diagD - linear_operator is 1D representation of diagonal
        if linear_operator is None and affine_offset is None:
            raise AffineError('linear_operator and affine_offset cannot '
                              'both be None')

        if sparse.issparse(affine_offset):
            #Convert sparse offset to an array
            self.affine_offset = affine_offset.toarray().reshape(-1)
        else:
            self.affine_offset = affine_offset
        self.linear_operator = linear_operator

        if linear_operator is None:
            self.noneD = True
            self.sparseD = False
            self.affineD = False
            self.diagD = False
            self.input_shape = affine_offset.shape
            self.output_shape = affine_offset.shape
        else:
            self.noneD = False
            self.sparseD = sparse.isspmatrix(self.linear_operator)
            self.sparseD_csr = sparse.isspmatrix_csr(self.linear_operator)
            if self.sparseD and not self.sparseD_csr:
                warnings.warn("Linear operator matrix is sparse, but not csr_matrix. Convert to csr_matrix for faster multiplications!")
            if self.sparseD_csr:
                self.linear_operator_T = sparse.csr_matrix(self.linear_operator.T)


            # does it support the affine_transform API
            if np.alltrue([hasattr(self.linear_operator, n) for 
                           n in ['linear_map',
                                 'affine_map',
                                 'affine_offset',
                                 'adjoint_map',
                                 'input_shape',
                                 'output_shape']]):
                self.input_shape = self.linear_operator.input_shape
                self.output_shape = self.linear_operator.output_shape
                self.affineD = True
                self.diagD = False
            elif linear_operator.ndim == 1 and not diag:
                self.linear_operator = self.linear_operator.reshape((1,-1))
                self.diagD = False
                self.affineD = False
                self.input_shape = (self.linear_operator.shape[1],)
                self.output_shape = (1,)
            elif linear_operator.ndim == 1 and diag:
                self.diagD = True
                self.affineD = False
                self.input_shape = (linear_operator.shape[0],)
                self.output_shape = (linear_operator.shape[0],)
            elif (input_shape is not None) and (len(input_shape) == 2):
                #Input coefficients is a matrix
                self.input_shape = input_shape
                self.output_shape = (linear_operator.shape[0], input_shape[1])
                self.diagD = False
                self.affineD = False
            else:
                self.input_shape = (linear_operator.shape[1],)
                self.output_shape = (linear_operator.shape[0],)
                self.diagD = False
                self.affineD = False

    def linear_map(self, x):
        r"""Apply linear part of transform to `x`

        Return :math:`Dx`

        Parameters
        ----------
        x : ndarray
            array to which to apply transform.  Can be 1D or 2D

        Returns
        -------
        Dx : ndarray
            `x` transformed with linear component

        Notes
        -----
        This routine is subclassed in affine_atom as a matrix multiplications,
        but could also call FFTs if D is a DFT matrix, in a subclass.
        """
        if self.noneD:
            return x
        elif self.affineD:
            return self.linear_operator.linear_map(x)
        elif self.sparseD:
            return self.linear_operator * x
        elif self.diagD:
            # Deal with 1D or 2D input or linear operator
            return broadcast_first(self.linear_operator, x, mul)
        return self.linear_operator.dot(x)

    def dot(self, x):
        r"""Apply linear part of transform to `x`.
        Returns `self.linear_map(x)` unless `x`
        is a transform, in which case it returns the composition.
        
        Parameters
        ----------
        x : ndarray
            array to which to apply transform.  Can be 1D or 2D

        Returns
        -------
        Dx : ndarray
            `x` transformed with linear component

        """
        if not isinstance(x, affine_transform):
            return self.linear_map(x)
        else:
            return composition(self, x)
    
    @property
    def T(self, doc="Return the adjoint."):
        return adjoint(self)

    @property
    def ndim(self, doc="Dimension of array."):
        return len(self.input_shape) + len(self.output_shape)

    @property
    def shape(self, doc="Shape of linear map. " + 
              "Usual 2-tuple if transform can be represented by 2-dim array."):
        if len(self.input_shape) == 1 and len(self.output_shape) == 1:
            return (self.output_shape[0], self.input_shape[0])
        else:
            return (self.output_shape, self.input_shape)

    def affine_map(self, x):
        r"""Apply linear and affine offset to `x`

        Return :math:`Dx+\alpha`

        Parameters
        ----------
        x : ndarray
            array to which to apply transform.  Can be 1D or 2D
        copy : {True, False}, optional
            If True, in situations where return is identical to `x`, ensure
            returned value is a copy.

        Returns
        -------
        Dx_a : ndarray
            `x` transformed with linear and offset components

        Notes
        -----
        This routine is subclassed in affine_atom as a matrix multiplications,
        but could also call FFTs if D is a DFT matrix, in a subclass.
        """
        if self.affineD:
            v = self.linear_operator.affine_map(x)
        else:
            v = self.linear_map(x)
        if self.affine_offset is not None:
            # Deal with 1D and 2D input, affine_offset cases
            return broadcast_first(self.affine_offset, v, add)
        return v

    def adjoint_map(self, u):
        r"""Apply transpose of linear component to `u`

        Return :math:`D^Tu`

        Parameters
        ----------
        u : ndarray
            array to which to apply transposed linear part of transform. Can be
            1D or 2D array
        copy : {True, False}, optional
            If True, in situations where return is identical to `u`, ensure
            returned value is a copy.

        Returns
        -------
        DTu : ndarray
            `u` transformed with transpose of linear component

        Notes
        -----
        This routine is currently a matrix multiplication, but could
        also call FFTs if D is a DFT matrix, in a subclass.
        """
        if self.noneD:
            return u
        if self.sparseD_csr:
            return self.linear_operator_T * u
        if self.sparseD:
            return self.linear_operator.T * u
        if self.diagD:
            # Deal with 1D or 2D input or linear operator
            return broadcast_first(self.linear_operator, u, mul)
        if self.affineD:
            return self.linear_operator.adjoint_map(u)
        return np.dot(self.linear_operator.T, u)


class linear_transform(affine_transform):
    """ A linear transform is an affine transform with no affine offset
    """
    def __init__(self, linear_operator, diag=False, input_shape=None):
        if linear_operator is None:
            raise AffineError('linear_operator cannot be None')
        affine_transform.__init__(self, linear_operator, None, diag=diag, input_shape=input_shape)


class selector(linear_transform):

    """
    Apply an affine transform after applying an
    indexing operation to the array.

    >>> from regreg.affine import selector, affine_transform
    >>> X = np.arange(30).reshape((6,5))
    >>> offset = np.arange(6)
    >>> transform = affine_transform(X, offset)
    >>> apply_to_first5 = selector(slice(0,5), (7,), transform)
    >>> apply_to_first5.linear_map(np.arange(7))
    array([ 30,  80, 130, 180, 230, 280])
    >>> np.dot(X, np.arange(5))
    array([ 30,  80, 130, 180, 230, 280])

    >>> apply_to_first5.affine_map(np.arange(7))
    array([ 30,  81, 132, 183, 234, 285])
    >>> np.dot(X, np.arange(5)) + offset
    array([ 30,  81, 132, 183, 234, 285])

    >>> result = np.array([275.,  290.,  305.,  320.,  335.,    0.,    0.])
    >>> np.testing.assert_allclose(result, apply_to_first5.adjoint_map(np.arange(6)))

    """

    def __init__(self,
                 index_obj,
                 initial_shape,
                 affine_transform=None,
                 reshape=None):

        self.index_obj = index_obj
        self.initial_shape = initial_shape
        self.reshape = reshape
        if affine_transform == None:
            test = np.empty(initial_shape)
            affine_transform = identity(test[index_obj].shape)
        self.affine_transform = affine_transform
        self.affine_offset = self.affine_transform.affine_offset
        self.input_shape = initial_shape
        if self.reshape is None:
            self.output_shape = self.affine_transform.output_shape
        else:
            self.output_shape = self.reshape
            
    def linear_map(self, x):
        x_indexed = x[self.index_obj]
        if self.reshape is not None:
            x_indexed.shape = self.reshape
        return self.affine_transform.linear_map(x_indexed)

    def affine_map(self, x):
        x_indexed = x[self.index_obj]
        if self.reshape is not None:
            x_indexed.shape = self.reshape
        return self.affine_transform.affine_map(x_indexed)

    def adjoint_map(self, u):
        if not hasattr(self, "_output"):
            self._output = np.zeros(self.initial_shape)
        if self.reshape is not None:
            u.shape = self.affine_transform.output_shape
        self._output[self.index_obj] = self.affine_transform.adjoint_map(u)
        return self._output

class scaler(linear_transform):

    def __init__(self, scalings):
        self.scalings = np.asarray(scalings)
        self.input_shape = self.output_shape = self.scalings.shape

    def linear_map(self, x):
        return self.scalings * x

    def affine_map(self, x):
        return self.linear_map(x)

    def adjoint_map(self, u):
        return self.linear_map(u)

class reshape(linear_transform):

    """
    Reshape the output of an affine transform.

    """

    def __init__(self, input_shape, output_shape):
        self.input_shape = input_shape
        self.output_shape = output_shape

    def linear_map(self, x):
        return x.reshape(self.output_shape)

    def affine_map(self, x):
        return self.linear_map(x)

    def adjoint_map(self, u):
        return u.reshape(self.input_shape)

# def tensor(T, first_input_index):
#     input_shape = T.shape[first_input_index:]
#     output_shape = T.shape[:first_input_index]

#     Tm = T.reshape((np.product(output_shape),
#                     np.product(input_shape)))
#     reshape_input = reshape(input_shape, Tm.shape[1])
#     reshape_output = reshape(Tm.shape[0], output_shape)
#     return composition(reshape_output, Tm, reshape_input)

class normalize(affine_transform):

    '''
    Normalize column by means and possibly scale. Could make
    a class for row normalization to.

    Columns are normalized to have std equal to value.
    '''

    def __init__(self, M, center=True, scale=True, value=1, inplace=False,
                 intercept_column=None):
        '''
        Parameters
        ----------
        M : ndarray or scipy.sparse
            The matrix to be normalized. If an ndarray and inplace=True,
            then the values of M are modified in place. Sparse matrices
            are not modified in place.

        center : bool
            Center the columns?

        scale : bool
            Scale the columns?

        value : float
            Set the std of the columns to be value.

        inplace : bool
            If sensible, modify values in place. For a sparse matrix,
            this will raise an exception if True and center==True.

        intercept_column : [int,None]
            Which column is the intercept if any? This column is
            not centered or scaled.

        '''
        n, p = M.shape
        self.value = value
        self.output_shape = (n,)
        self.input_shape = (p,)

        self.sparseM = sparse.isspmatrix(M)
        self.intercept_column = intercept_column
        self.M = M

        self.center = center
        self.scale = scale

        self.inplace = inplace

        if value != 1 and not scale:
            warnings.warn('setting length of columns when not being asked to scale them')

        # we divide by n instead of n-1 in the scalings
        # so that np.std is constant
        
        if self.center:
            if self.inplace and self.sparseM:
                raise ValueError('resulting matrix will not be sparse if centering performed inplace')

            if not self.sparseM:
                col_means = M.mean(0)
            else:
                tmp = M.copy()
                col_means = np.asarray(tmp.mean(0)).reshape(-1)
                tmp.data **= 2

            if self.intercept_column is not None:
                col_means[self.intercept_column] = 0

            if self.scale:
                if not self.sparseM:
                    self.col_stds = np.sqrt((np.sum(M**2,0) - n * col_means**2) / n) / np.sqrt(self.value)
                else:
                    self.col_stds = np.asarray(np.sqrt((np.asarray(tmp.sum(0)).reshape(-1) - n * col_means**2) / n) / np.sqrt(self.value)).reshape(-1)
                if self.intercept_column is not None:
                    self.col_stds[self.intercept_column] = 1. 

            if not self.sparseM and self.inplace:
                self.M -= col_means[np.newaxis,:]
                if self.scale:
                    self.M /= self.col_stds[np.newaxis,:]
                    # if scaling has been applied in place, 
                    # no need to do it again
                    self.col_stds = None
                    self.scale = False
        elif self.scale:
            if not self.sparseM:
                self.col_stds = np.sqrt(np.sum(M**2,0) / n) / np.sqrt(self.value)
            else:
                tmp = M.copy()
                tmp.data **= 2
                self.col_stds = np.asarray(np.sqrt((tmp.sum(0)) / n) / np.sqrt(self.value)).reshape(-1)
            if self.intercept_column is not None:
                self.col_stds[self.intercept_column] = 1.
            if self.inplace:
                self.M /= self.col_stds[np.newaxis,:]
                # if scaling has been applied in place, 
                # no need to do it again
                self.col_stds = None
                self.scale = False
        self.affine_offset = None
        
    def linear_map(self, x):
        if self.intercept_column is not None:
            x_intercept = x[self.intercept_column]
        if self.scale:
            if x.ndim == 1:
                x = x / self.col_stds
            elif x.ndim == 2:
                x = x / self.col_stds[:,np.newaxis]
            else:
                raise ValueError('normalize only implemented for 1D and 2D inputs')
        if self.sparseM:
            if x.ndim == 1:
                v = np.asarray(self.M * (x.reshape((-1,1)))).reshape(self.output_shape)
            else:
                v = np.asarray(self.M * x).reshape((self.output_shape[0], x.shape[1]))
        else:
            v = np.dot(self.M, x)
        if self.center:
            if x.ndim == 1:
                v -= v.mean()
            elif x.ndim == 2:
                v -= v.mean(0)[np.newaxis,:]
            else:
                raise ValueError('normalize only implemented for 1D and 2D inputs')
        if self.intercept_column is not None and self.center:
            if x.ndim == 2:
                v += x_intercept[np.newaxis,:]
            else:
                v += x_intercept
        return v

    def affine_map(self, x):
        return self.linear_map(x)

    def adjoint_map(self, u):
        if u.ndim == 1:
            u_mean = u.mean()
        elif u.ndim == 2:
            u_mean = u.mean(0)
        if self.center:
            if u.ndim == 1:
                u = u - u_mean
            elif u.ndim == 2:
                if self.center:
                    u = u - u_mean[np.newaxis,:]
            else:
                raise ValueError('normalize only implemented for 1D and 2D inputs')
        if self.sparseM:
            if u.ndim == 1:
                v = np.asarray((u.T * self.M).T).reshape(-1)
            else:
                v = np.asarray((u.T * self.M).T)
        else:
            v = np.dot(u.T, self.M).T

        if self.scale:
            if u.ndim == 1:
                v /= self.col_stds
            elif u.ndim == 2:
                v /= self.col_stds[:,None]
        if self.intercept_column is not None and (self.center or self.scale):
            v[self.intercept_column] = u_mean * u.shape[0]
        return v

    def slice_columns(self, index_obj):
        """

        Parameters
        ----------
        index_obj: slice, list, np.bool
            An object on which to index the columns of self.M.
            Must be a slice object or list so scipy.sparse matrices
            can be sliced.

        Returns
        -------
        n : normalize
            A transform which agrees with self having zeroed out
            all coefficients not captured by index_obj.

        Notes
        -----
        This method does not check whether or not ``self.intercept_column`` is
        None so it must be set by hand on the returned instance.

        Examples
        --------
        >>> from regreg.affine import normalize
        >>> X = np.array([1.2,3.4,5.6,7.8,1.3,4.5,5.6,7.8,1.1,3.4])
        >>> D = np.identity(X.shape[0]) - np.diag(np.ones(X.shape[0]-1),1)
        >>> nD = normalize(D)
        >>> X_sliced = X.copy()
        >>> X_sliced[:4] = 0; X_sliced[6:] = 0
        >>> expected = [0, 0, 0, -2.906888, -7.155417, 10.06230, 0, 0, 0, 0]
        >>> np.allclose(nD.linear_map(X_sliced), expected)
        True
        >>> nD_slice = nD.slice_columns(slice(4,6))
        >>> np.allclose(nD_slice.linear_map(X[slice(4,6)]), expected)
        True
        """
        if type(index_obj) not in [type(slice(0,4)), type([])]:
            # try to find nonzero indices if a boolean array
            if index_obj.dtype == np.bool:
                index_obj = np.nonzero(index_obj)[0]

        new_obj = normalize.__new__(normalize)
        new_obj.sparseM = self.sparseM

        # explicitly assumes there is no intercept column
        if self.intercept_column is not None:
            if self.intercept_column not in list(index_obj):
                new_obj.intercept_column = None
            else:
                new_obj.intercept_column = list(index_obj).index(self.intercept_column)
        else:
            new_obj.intercept_column = None
        new_obj.value = self.value
        try:
            new_obj.M = self.M[:,index_obj]
        except TypeError: # sparse matrix is of wrong format
            new_obj.M = self.M.tolil()[:,index_obj].tocsc()

        new_obj.input_shape = (new_obj.M.shape[1],)
        new_obj.output_shape = (self.M.shape[0],)
        new_obj.scale = self.scale
        new_obj.center = self.center
        if self.scale:
            new_obj.col_stds = self.col_stds[index_obj]
        new_obj.affine_offset = self.affine_offset
        return new_obj

    def normalized_array(self):
        if self.inplace:
            return self.M
        else:
            raise ValueError('only possible to extract matrix if normalization was done inplace')

class identity(affine_transform):

    """
    Identity transform
    """

    def __init__(self, input_shape):
        self.input_shape = self.output_shape = input_shape
        self.affine_offset = None
        self.linear_operator = None

    def affine_map(self, x):
        return self.linear_map(x)

    def linear_map(self, x):
        return x

    def adjoint_map(self, x):
        return self.linear_map(x)

class vstack(affine_transform):
    """
    Stack several affine transforms vertically together though
    not necessarily as a big matrix.
    """

    def __init__(self, transforms):
        self.input_shape = -1
        self.output_shapes = []
        self.transforms = []
        self.output_slices = []
        total_output = 0
        for transform in transforms:
            transform = astransform(transform)
            if self.input_shape == -1:
                self.input_shape = transform.input_shape
            else:
                if transform.input_shape != self.input_shape:
                    raise ValueError("input dimensions don't agree")
            self.transforms.append(transform)
            self.output_shapes.append(transform.output_shape)
            increment = np.product(transform.output_shape)
            self.output_slices.append(slice(total_output, total_output + increment))
            total_output += increment

        self.output_shape = (total_output,)
        self.group_dtype = np.dtype([('group_%d' % i, np.float, shape) 
                                     for i, shape in enumerate(self.output_shapes)])
        self.output_groups = self.group_dtype.names 

        # figure out the affine offset
        self.affine_offset = np.empty(self.output_shape)
        x = np.zeros(self.input_shape)
        for g, t in zip(self.output_slices, self.transforms):
            self.affine_offset[g] = t.affine_map(x)
        if np.all(np.equal(self.affine_offset, 0)):
            self.affine_offset = None
            
    def linear_map(self, x):
        result = np.empty(self.output_shape)
        for g, t in zip(self.output_slices, self.transforms):
            result[g] = t.linear_map(x)
        return result

    def affine_map(self, x):
        result = np.empty(self.output_shape)
        for g, t in zip(self.output_slices, self.transforms):
            result[g] = t.linear_map(x)
        if self.affine_offset is not None:
            return result + self.affine_offset
        else:
            return result

    def adjoint_map(self, u):
        result = np.zeros(self.input_shape)
        for g, t, s in zip(self.output_slices, self.transforms,
                           self.output_shapes):
            result += t.adjoint_map(u[g].reshape(s))
        return result

class hstack(affine_transform):
    """
    Stack several affine transforms horizontally together though
    not necessarily as a big matrix.
   
    """

    def __init__(self, transforms):
        self.output_shape = -1
        self.input_shapes = []
        self.transforms = []
        self.input_slices = []
        total_input = 0
        for transform in transforms:
            transform = astransform(transform)
            if self.output_shape == -1:
                self.output_shape = transform.output_shape
            else:
                if transform.output_shape != self.output_shape:
                    raise ValueError("output dimensions don't agree")
            self.transforms.append(transform)
            self.input_shapes.append(transform.input_shape)
            increment = np.product(transform.input_shape)
            self.input_slices.append(slice(total_input, total_input + increment))
            total_input += increment

        self.input_shape = (total_input,)
        self.group_dtype = np.dtype([('group_%d' % i, np.float, shape) 
                                     for i, shape in enumerate(self.input_shapes)])
        self.input_groups = self.group_dtype.names 

        # figure out the affine offset
        self.affine_offset = np.zeros(self.output_shape)
        for g, s, t in zip(self.input_slices, self.input_shapes,
                           self.transforms):
            self.affine_offset += t.affine_map(np.zeros(s))
        if np.all(np.equal(self.affine_offset, 0)):
            self.affine_offset = None

    def linear_map(self, x):
        result = np.zeros(self.output_shape)
        for g, t, s in zip(self.input_slices, self.transforms,
                           self.input_shapes):
            result += t.linear_map(x[g].reshape(s))
        return result

    def affine_map(self, x):
        result = np.zeros(self.output_shape)
        for g, t, s in zip(self.input_slices, self.transforms,
                        self.input_shapes):
            result += t.linear_map(x[g].reshape(s))
        if self.affine_offset is not None:
            return result + self.affine_offset
        else:
            return result

    def adjoint_map(self, u):
        result = np.empty(self.input_shape)
        #XXX this reshaping will fail for shapes that aren't
        # 1D, would have to view as self.group_dtype to
        # take advantange of different shapes
        for g, t, s in zip(self.input_slices, self.transforms,
                           self.input_shapes):
            result[g] = t.adjoint_map(u).reshape(-1)
        return result

class product(affine_transform):
    """
    Create a transform that maps the product of the inputs
    to the product of the outputs.
   
    """

    def __init__(self, transforms):
        self.output_shapes = []
        self.input_shapes = []
        self.transforms = []
        self.input_slices = []
        self.output_slices = []
        total_input = 0
        total_output = 0
        for transform in transforms:
            transform = astransform(transform)
            self.transforms.append(transform)
            self.input_shapes.append(transform.input_shape)
            self.output_shapes.append(transform.output_shape)
            input_increment = np.product(transform.input_shape)
            output_increment = np.product(transform.output_shape)
            self.input_slices.append(slice(total_input, total_input + input_increment))
            self.output_slices.append(slice(total_output, total_output + output_increment))
            total_input += input_increment
            total_output += output_increment

        self.input_shape = (total_input,)
        self.output_shape = (total_output,)
        self.input_group_dtype = np.dtype([('group_%d' % i, np.float, shape) 
                                           for i, shape in enumerate(self.input_shapes)])
        self.input_groups = self.input_group_dtype.names 

        self.output_group_dtype = np.dtype([('group_%d' % i, np.float, shape) 
                                           for i, shape in enumerate(self.input_shapes)])
        self.output_groups = self.output_group_dtype.names 

        # figure out the affine offset
        self.affine_offset = np.zeros(self.output_shape)
        for g, s, t in zip(self.output_slices, self.input_shapes,
                           self.transforms):
            self.affine_offset[g] = t.affine_map(np.zeros(s))
        if np.all(np.equal(self.affine_offset, 0)):
            self.affine_offset = None

    def linear_map(self, x):
        result = np.zeros(self.output_shape)
        for og, ig, t, s in zip(self.output_slices,
                                self.input_slices, 
                                self.transforms,
                                self.input_shapes):
            result[og] = t.linear_map(x[ig].reshape(s))
        return result

    def affine_map(self, x):
        result = np.zeros(self.output_shape)
        for og, ig, t, s in zip(self.output_slices,
                                self.input_slices, 
                                self.transforms,
                                self.input_shapes):
            result[og] = t.linear_map(x[ig].reshape(s))
        if self.affine_offset is not None:
            return result + self.affine_offset
        else:
            return result

    def adjoint_map(self, u):
        result = np.empty(self.input_shape)
        for og, ig, t, s in zip(self.output_slices,
                                self.input_slices, 
                                self.transforms,
                                self.input_shapes):
            result[ig] = t.adjoint_map(u[og]).reshape(-1)
        return result


def power_L(transform, max_its=500,tol=1e-8, debug=False):
    """
    Approximate the largest singular value (squared) of the linear part of
    a transform using power iterations
    
    TODO: should this be the largest singular value instead (i.e. not squared?)
    """

    transform = astransform(transform)
    v = np.random.standard_normal(transform.input_shape)
    old_norm = 0.
    norm = 1.
    itercount = 0
    while np.fabs(norm-old_norm)/norm > tol and itercount < max_its:
        v = transform.adjoint_map(transform.linear_map(v))
        old_norm = norm
        norm = np.linalg.norm(v)
        v /= norm
        if debug:
            print("L", norm)
        itercount += 1
    return norm

def astransform(X):
    """
    If X is an affine_transform, return X,
    else try to cast it as an affine_transform
    """
    if isinstance(X, affine_transform):
        return X
    else:
        return linear_transform(X)

class aslinear(linear_transform):
    """
    Return only linear part of affine transform
    """
    def __init__(self, transform):
        self._transform = astransform(transform)
        self.affine_offset = None
        self.input_shape = self._transform.output_shape
        self.output_shape = self._transform.input_shape

    def linear_map(self, x):
        return self._transform.linear_map(x)

    def affine_map(self, x):
        return self.linear_map(x)

    def adjoint_map(self, x):
        return self._transform.adjoint_map(x)

class adjoint(linear_transform):

    """
    Given an affine_transform, return a linear_transform
    that is the adjoint of its linear part.
    """
    def __init__(self, transform):
        self.transform = astransform(transform)
        self.affine_offset = None
        self.input_shape = self.transform.output_shape
        self.output_shape = self.transform.input_shape

    def linear_map(self, x):
        return self.transform.adjoint_map(x)

    def affine_map(self, x):
        return self.linear_map(x)

    def adjoint_map(self, x):
        return self.transform.linear_map(x)

class tensorize(affine_transform):

    """
    Given an affine_transform, return a linear_transform
    that expects q copies of something with transform's input_shape.

    This class effectively makes explicit that a transform
    may expect a matrix rather than a single vector.

    """
    def __init__(self, transform, q):
        self.transform = astransform(transform)
        self.affine_offset = self.transform.affine_offset
        self.input_shape = self.transform.input_shape + (q,)
        self.output_shape = self.transform.output_shape + (q,)

    def linear_map(self, x):
        return self.transform.linear_map(x)

    def affine_map(self, x):
        v = self.linear_map(x) 
        if self.affine_offset is not None:
            return v + self.affine_offset[:, np.newaxis]
        return v

    def adjoint_map(self, x):
        return self.transform.adjoint_map(x)

class residual(affine_transform):

    """
    Compute the residual from an affine transform.
    """

    def __init__(self, transform):
        self.transform = astransform(transform)
        self.input_shape = self.transform.input_shape
        self.output_shape = self.transform.output_shape
        self.affine_offset = None
        if not self.input_shape == self.output_shape:
            raise ValueError('output and input shapes should be the same to compute residual')

    def linear_map(self, x):
        return x - self.transform.linear_map(x)

    def affine_map(self, x):
        return x - self.transform.affine_map(x)

    def adjoint_map(self, u):
        return u - self.transform.adjoint_map(u)

class composition(affine_transform):

    """
    Composes a list of affine transforms, executing right to left
    """

    def __init__(self, *transforms):
        self.transforms = [astransform(t) for t in transforms]
        self.input_shape = self.transforms[-1].input_shape
        self.output_shape = self.transforms[0].output_shape

        # compute the affine_offset
        affine_offset = self.affine_map(np.zeros(self.input_shape))
        if not np.allclose(affine_offset, 0): 
            self.affine_offset = None
        else:
            self.affine_offset = affine_offset

    def linear_map(self, x):
        output = x
        for transform in self.transforms[::-1]:
            output = transform.linear_map(output)
        return output

    def affine_map(self, x):
        output = x
        for transform in self.transforms[::-1]:
            output = transform.affine_map(output)
        return output

    def adjoint_map(self, x):
        output = x
        for transform in self.transforms:
            output = transform.adjoint_map(output)
        return output

class affine_sum(affine_transform):

    """
    Creates the (weighted) sum of a list of affine_transforms
    """

    def __init__(self, transforms, weights=None):
        self.transforms = [astransform(T) for T in transforms]
        if weights is None:
            self.weights = np.ones(len(self.transforms))
        else:
            if not len(self.transforms) == len(weights):
                raise ValueError("Must specify a weight for each transform")
            self.weights = weights
        self.input_shape = self.transforms[0].input_shape
        self.output_shape = self.transforms[0].output_shape

        # compute the affine_offset
        affine_offset = self.affine_map(np.zeros(self.input_shape))
        if np.allclose(affine_offset, 0): 
            self.affine_offset = None
        else:
            self.affine_offset = affine_offset

    def linear_map(self, x):
        output = 0
        for transform, weight in zip(self.transforms[::-1], self.weights[::-1]):
            output += weight * transform.linear_map(x)
        return output

    def affine_map(self, x):
        output = 0
        for transform, weight in zip(self.transforms[::-1], self.weights[::-1]):
            output += weight * transform.affine_map(x)
        return output

    def adjoint_map(self, x):
        output = 0
        for transform, weight in zip(self.transforms[::-1], self.weights[::-1]):
            output += weight * transform.adjoint_map(x)
        return output


class scalar_multiply(affine_transform):

    def __init__(self, atransform, scalar):
        self.input_shape, self.output_shape = (atransform.input_shape, atransform.output_shape)
        self.scalar = scalar
        self.affine_offset = None
        self._atransform = atransform

    def affine_map(self, x):
        if self.scalar != 1.:
            return self._atransform.affine_map(x) * self.scalar
        else:
            return self._atransform.affine_map(x)

    def linear_map(self, x):
        if self.scalar != 1.:
            return self._atransform.linear_map(x) * self.scalar 
        else:
            return self._atransform.linear_map(x)


    def adjoint_map(self, x):
        if self.scalar != 1.:
            return self._atransform.adjoint_map(x) * self.scalar 
        else:
            return self._atransform.adjoint_map(x)

class posneg(affine_transform):

    def __init__(self, linear_transform):
        self.linear_transform = astransform(linear_transform)
        # where to store output so we don't recreate arrays 
        self.affine_offset = None
        self.input_shape = (2,) + self.linear_transform.input_shape
        self._adjoint_output = np.zeros(self.input_shape)
        self.output_shape = self.linear_transform.output_shape

    def linear_map(self, x):
        L = self.linear_transform.linear_map
        return  L(x[0]) - L(x[1])

    def affine_map(self, x):
        return self.linear_map(x)

    def adjoint_map(self, x):
        u = self._adjoint_output
        u[0] = self.linear_transform.adjoint_map(x)
        u[1] = -u[0]
        return u

def todense(transform):
    """
    Return a dense array representation of a transform -- use
    carefully -- it could be large.
    """
    if len(transform.input_shape) == 1:
        I = np.identity(np.product(transform.input_shape))
        return transform.linear_map(I)
    else:
        raise NotImplementedError('expecting a 1D shape as input')




import numpy as np
from scipy import sparse

from ..affine import astransform, affine_transform


def difference_transform(X, order=1, sorted=False,
                         transform=False):
    """
    Compute the divided difference matrix for X
    after sorting X.

    Parameters
    ----------

    X: np.array, np.float, ndim=1
        X coordinates where discrete derivative is computed.

    order: int
        What order of difference should we compute?

    sorted: bool
        Is X sorted?

    transform: bool
        If True, return a linear_transform rather
        than a sparse matrix.

    Returns
    -------

    D: np.array, ndim=2, shape=(n-order,order)
        Matrix of divided differences of sorted X.

    """
    if not sorted:
        X = np.sort(X)
    X = np.asarray(X)
    n = X.shape[0]
    Dfinal = np.identity(n)
    for j in range(1, order+1):
        D = (-np.identity(n-j+1)+np.diag(np.ones(n-j),k=1))[:-1]
        steps = X[j:]-X[:-j]
        inv_steps = np.zeros(steps.shape)
        inv_steps[steps != 0] = 1. / steps[steps != 0]
        D = np.dot(np.diag(inv_steps), D)
        Dfinal = np.dot(D, Dfinal)
    if not transform:
        return sparse.csr_matrix(Dfinal)
    return astransform(Dfinal)

class trend_filter(affine_transform):

    def __init__(self, knots, order=1, sorted=False):
        self.order = order
        self.knots = np.sort(knots)
        self.steps = self.knots[1:] - self.knots[:-1]

        self.linear_transform = difference_transform(knots, order=order, sorted=True,
                                                     transform=True)
        self.affine_offset = None
        self.input_shape = self.linear_transform.input_shape
        self.output_shape = self.linear_transform.output_shape

    @classmethod
    def grid(cls, m, order=1, sorted=False):
        return cls(np.arange(m), order=order, sorted=sorted)

    def linear_map(self, x):
        return self.linear_transform.linear_map(x)

    def affine_map(self, x):
        return self.linear_map(x)

    def adjoint_map(self, x):
        return self.linear_transform.adjoint_map(x)


class trend_filter_inverse(affine_transform):

    def __init__(self, knots, order=1, sorted=False):
        self.order = order
        self.knots = np.sort(knots)
        self.steps = self.knots[1:] - self.knots[:-1]
        if order != 1:
            raise ValueError('pseudo inverse only worked out for first order')
            
        dtransform = difference_transform(knots, order=order, sorted=True,
                                          transform=True)

        self.affine_offset = None
        self.output_shape = dtransform.input_shape
        self.input_shape = dtransform.output_shape

    @classmethod
    def grid(cls, m, order=1, sorted=False):
        return cls(np.arange(m), order=order, sorted=sorted)

    def linear_map(self, x):
        if x.ndim == 1:
            v = np.zeros(self.output_shape)
            v[1:] = np.cumsum(x * self.steps)
            v -= v.mean()
            return v
        elif x.ndim == 2:
            # assuming m is the first axis
            v = np.zeros((self.output_shape[0], x.shape[1]))
            v[1:] = np.cumsum(x * self.steps[:,np.newaxis], axis=0)
            v -= v.mean(0)
            return v

    def affine_map(self, x):
        return self.linear_map(x)

    def adjoint_map(self, x):
        if x.ndim == 1:
            x = x - x.mean(0)
            C = np.cumsum(x[1:][::-1])[::-1]
            return C * self.steps
        if x.ndim == 2:
            # assuming m is the first axis
            x = x - x.mean(0)[np.newaxis,:]
            C = np.cumsum(x[1:][::-1], 1)[::-1]
            return C * self.steps[:,np.newaxis]





from __future__ import print_function, division, absolute_import

import numpy as np

from copy import copy
import scipy.optimize

import regreg.api as rr
from regreg.atoms.tests.test_seminorms import all_close
from regreg.tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_lasso():
    '''
    this test verifies that the l1 prox can be solved
    by a primal/dual specification 

    obviously, we don't to solve the l1 prox this way,
    but it verifies that specification is working correctly

    '''

    l1 = rr.l1norm(4, lagrange=2.)

    l11 = rr.l1norm(4, lagrange=1.)
    l12 = rr.l1norm(4, lagrange=1.)


    X = np.random.standard_normal((10,4))
    Y = np.random.standard_normal(10) + 3
    
    loss = rr.quadratic_loss.affine(X, -Y)
    p1 = rr.container(l11, loss, l12)

    solver1 = rr.FISTA(p1)
    solver1.fit(tol=1.0e-12, min_its=500)

    p2 = rr.separable_problem.singleton(l1, loss)
    solver2 = rr.FISTA(p2)
    solver2.fit(tol=1.0e-12)

    f = p2.objective
    ans = scipy.optimize.fmin_powell(f, np.zeros(4), ftol=1.0e-12)
    print(f(solver2.composite.coefs), f(ans))
    print(f(solver1.composite.coefs), f(ans))

    yield all_close, ans, solver2.composite.coefs, 'singleton solver', None
    yield all_close, solver1.composite.coefs, solver2.composite.coefs, 'container solver', None

@set_seed_for_test()
def test_quadratic_for_smooth():
    '''
    this test is a check to ensure that the quadratic part 
    of the smooth functions are being used in the proximal step
    '''

    L = 0.45

    W = np.random.standard_normal(40)
    Z = np.random.standard_normal(40)
    U = np.random.standard_normal(40)

    atomq = rr.identity_quadratic(0.4, U, W, 0)
    atom = rr.l1norm(40, quadratic=atomq, lagrange=0.12)

    # specifying in this way should be the same as if we put 0.5*L below
    loss = rr.quadratic_loss.shift(Z, coef=0.6*L)
    lq = rr.identity_quadratic(0.4*L, Z, 0, 0)
    loss.quadratic = lq 

    ww = np.random.standard_normal(40)

    # specifying in this way should be the same as if we put 0.5*L below
    loss2 = rr.quadratic_loss.shift(Z, coef=L)
    yield all_close, loss2.objective(ww), loss.objective(ww), 'checking objective', None

    yield all_close, lq.objective(ww, 'func'), loss.nonsmooth_objective(ww), 'checking nonsmooth objective', None
    yield all_close, loss2.smooth_objective(ww, 'func'), 0.5 / 0.3 * loss.smooth_objective(ww, 'func'), 'checking smooth objective func', None
    yield all_close, loss2.smooth_objective(ww, 'grad'), 0.5 / 0.3 * loss.smooth_objective(ww, 'grad'), 'checking smooth objective grad', None

    problem = rr.container(loss, atom)
    solver = rr.FISTA(problem)
    solver.fit(tol=1.0e-12)

    problem3 = rr.simple_problem(loss, atom)
    solver3 = rr.FISTA(problem3)
    solver3.fit(tol=1.0e-12, coef_stop=True)

    loss4 = rr.quadratic_loss.shift(Z, coef=0.6*L)
    problem4 = rr.simple_problem(loss4, atom)
    problem4.quadratic = lq
    solver4 = rr.FISTA(problem4)
    solver4.fit(tol=1.0e-12)

    gg_soln = rr.gengrad(problem, L)

    loss6 = rr.quadratic_loss.shift(Z, coef=0.6*L)
    loss6.quadratic = lq + atom.quadratic
    atomcp = copy(atom)
    atomcp.quadratic = rr.identity_quadratic(0,0,0,0)
    problem6 = rr.dual_problem(loss6.conjugate, rr.identity(loss6.shape), atomcp.conjugate)
    problem6.lipschitz = L + atom.quadratic.coef
    dsoln2 = problem6.solve(coef_stop=True, tol=1.e-10, 
                            max_its=100)

    problem2 = rr.container(loss2, atom)
    solver2 = rr.FISTA(problem2)
    solver2.fit(tol=1.0e-12, coef_stop=True)

    q = rr.identity_quadratic(L, Z, 0, 0)

    yield all_close, problem.objective(ww), atom.nonsmooth_objective(ww) + q.objective(ww,'func'), '', None

    atom = rr.l1norm(40, quadratic=atomq, lagrange=0.12)
    aq = atom.solve(q)
    for p, msg in zip([solver3.composite.coefs,
                       gg_soln,
                       solver2.composite.coefs,
                       dsoln2,
                       solver.composite.coefs,
                       solver4.composite.coefs],
                      ['simple_problem with loss having no quadratic',
                       'gen grad',
                       'container with loss having no quadratic',
                       'dual problem with loss having a quadratic',
                       'container with loss having a quadratic',
                       'simple_problem having a quadratic']):
        yield all_close, aq, p, msg, None

@set_seed_for_test()
def test_quadratic_for_smooth2():
    '''
    this test is a check to ensure that the
    quadratic part of the smooth functions are being used in the proximal step

    '''

    L = 2

    W = np.arange(5)
    Z = 0.5 * np.arange(5)[::-1]
    U = 1.5 * np.arange(5)

    atomq = rr.identity_quadratic(0.4, U, W, 0)
    atom = rr.l1norm(5, quadratic=atomq, lagrange=0.1)

    # specifying in this way should be the same as if we put 0.5*L below
    loss = rr.quadratic_loss.shift(Z, coef=0.6*L)
    lq = rr.identity_quadratic(0.4*L, Z, 0, 0)
    loss.quadratic = lq 

    ww = np.ones(5)

    # specifying in this way should be the same as if we put 0.5*L below
    loss2 = rr.quadratic_loss.shift(Z, coef=L)
    np.testing.assert_allclose(loss2.objective(ww), loss.objective(ww))
    np.testing.assert_allclose(lq.objective(ww, 'func'), loss.nonsmooth_objective(ww))
    np.testing.assert_allclose(loss2.smooth_objective(ww, 'func'), 0.5 / 0.3 * loss.smooth_objective(ww, 'func'))
    np.testing.assert_allclose(loss2.smooth_objective(ww, 'grad'), 0.5 / 0.3 * loss.smooth_objective(ww, 'grad'))

    problem = rr.container(loss, atom)
    solver = rr.FISTA(problem)
    solver.fit(tol=1.0e-12)

    problem3 = rr.simple_problem(loss, atom)
    solver3 = rr.FISTA(problem3)
    solver3.fit(tol=1.0e-12, coef_stop=True)

    loss4 = rr.quadratic_loss.shift(Z, coef=0.6*L)
    problem4 = rr.simple_problem(loss4, atom)
    problem4.quadratic = lq
    solver4 = rr.FISTA(problem4)
    solver4.fit(tol=1.0e-12)

    gg_soln = rr.gengrad(problem, L)

    loss6 = rr.quadratic_loss.shift(Z, coef=0.6*L)
    loss6.quadratic = lq + atom.quadratic
    atomcp = copy(atom)
    atomcp.quadratic = rr.identity_quadratic(0,0,0,0)
    problem6 = rr.dual_problem(loss6.conjugate, rr.identity(loss6.shape), atomcp.conjugate)
    problem6.lipschitz = L + atom.quadratic.coef
    dsoln2 = problem6.solve(coef_stop=True, tol=1.e-10, 
                            max_its=100)

    problem2 = rr.container(loss2, atom)
    solver2 = rr.FISTA(problem2)
    solver2.fit(tol=1.0e-12, coef_stop=True)

    q = rr.identity_quadratic(L, Z, 0, 0)

    yield all_close, problem.objective(ww), atom.nonsmooth_objective(ww) + q.objective(ww,'func'), '', None

    aq = atom.solve(q)
    for p, msg in zip([solver3.composite.coefs,
                       gg_soln,
                       solver2.composite.coefs,
                       dsoln2,
                       solver.composite.coefs,
                       solver4.composite.coefs],
                      ['simple_problem with loss having no quadratic',
                       'gen grad',
                       'container with loss having no quadratic',
                       'dual problem with loss having a quadratic',
                       'container with loss having a quadratic',
                       'simple_problem having a quadratic']):
        yield all_close, aq, p, msg, None



import numpy as np

from copy import copy
import scipy.optimize

import regreg.api as rr
from regreg.tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_quadratic():

    l = rr.quadratic_loss(5, coef=3., offset=np.arange(5))
    l.quadratic = rr.identity_quadratic(1, np.ones(5), 2*np.ones(5), 3.)
    c1 = l.get_conjugate()

    q1 = rr.identity_quadratic(3, np.arange(5), 0, 0)
    q2 = q1 + l.quadratic
    c2 = rr.zero(5, quadratic=q2.collapsed()).conjugate

    ww = np.random.standard_normal(5)
    np.testing.assert_almost_equal(c2.smooth_objective(ww, 'grad'),
                                   c1.smooth_objective(ww, 'grad'))

    np.testing.assert_almost_equal(c2.objective(ww),
                                   c1.objective(ww))

    np.testing.assert_almost_equal(c2.smooth_objective(ww, 'func') + 
                                   c2.nonsmooth_objective(ww),
                                   c1.smooth_objective(ww, 'func') + 
                                   c1.nonsmooth_objective(ww))




"""
Implementation of the sparse group LASSO atom.

This penalty is defined by groups and individual feature weights and is
$$
\beta \mapsto \sum_j \lambda_j |\beta_j| + \sum_g \alpha_g \|\beta_g\|_2
$$

"""

from warnings import warn
from copy import copy
import numpy as np
from scipy import sparse

from .seminorms import seminorm
from .group_lasso import group_lasso
from .weighted_atoms import l1norm as weighted_l1norm
from ..atoms import _work_out_conjugate
from ..objdoctemplates import objective_doc_templater
from ..doctemplates import (doc_template_user, doc_template_provider)
from ..problems.composite import smooth_conjugate

@objective_doc_templater()
class sparse_group_lasso(group_lasso, seminorm):

    """
    Sparse group LASSO
    """

    objective_template = (r""" \left[ \sum_j \alpha_j |%(var)s_j|""" + 
                          r""" + \sum_g \lambda_g \|%(var)s[g]\|_2 \right]""")
    objective_vars = group_lasso.objective_vars.copy()
    objective_vars['normklass'] = 'sparse_group_lasso'
    objective_vars['dualnormklass'] = 'sparse_group_lasso_dual'
    objective_vars['initargs'] = '[1, 1, 2, 2, 2], [0, 0.5, 0.2, 0.2, 0.2]'

    def __init__(self, 
                 groups,
                 lasso_weights, 
                 weights={},
                 lagrange=None, 
                 bound=None, 
                 offset=None, 
                 quadratic=None,
                 initial=None):

         group_lasso.__init__(self, 
                              groups, 
                              weights=weights,
                              lagrange=lagrange,
                              bound=bound,
                              offset=offset,
                              quadratic=quadratic,
                              initial=initial)

         self.lasso_weights = np.asarray(lasso_weights)
         if self.lasso_weights.shape != self.groups.shape:
             self.lasso_weights = self.lasso_weights * np.ones(self.shape)
         self._weighted_l1norm = weighted_l1norm(self.lasso_weights, 
                                                 lagrange=lagrange,
                                                 bound=bound)
         self._weighted_supnorm = self._weighted_l1norm.conjugate

    @doc_template_user
    def seminorm(self, x, lagrange=None, check_feasibility=False):

        lagrange = seminorm.seminorm(self, x, 
                                     check_feasibility=check_feasibility, 
                                     lagrange=lagrange)

        val = group_lasso.seminorm(self, 
                                   x,
                                   lagrange=lagrange,
                                   check_feasibility=check_feasibility)

        val += self._weighted_l1norm.seminorm(x,
                                              lagrange=lagrange,
                                              check_feasibility=check_feasibility)

        return val

    @doc_template_user
    def constraint(self, x, bound=None):
         bound = seminorm.constraint(self, x, bound=bound)
         inbox = self.seminorm(x, 
                               lagrange=1,
                               check_feasibility=True) <= bound * (1+self.tol)
         if inbox:
             return 0
         else:
             return np.inf

    @doc_template_user
    def lagrange_prox(self, x,  lipschitz=1, lagrange=None):
         lagrange = seminorm.lagrange_prox(self, x, lipschitz, lagrange)

         # this gives residual from projection onto the 
         # weighted supnorm ball

         initial_proj = self._weighted_l1norm.lagrange_prox(x, 
                                                            lipschitz, 
                                                            lagrange)

         # each group component is then appropriately
         # group lasso soft-thresholded

         final_proj = group_lasso.lagrange_prox(self, 
                                                initial_proj,
                                                lipschitz,
                                                lagrange)

         return final_proj

    @doc_template_user
    def bound_prox(self, x, bound=None):
        raise NotImplementedError

    def __copy__(self):
         return self.__class__(self.groups.copy(),
                               self.lasso_weights.copy(),
                               weights=copy(self.weights),
                               quadratic=self.quadratic,
                               initial=self.coefs,
                               bound=copy(self.bound),
                               lagrange=copy(self.lagrange),
                               offset=copy(self.offset))

    def terms(self, arg):
        """
        Return the args that are summed
        in computing the seminorm.

        >>> import regreg.api as rr
        >>> groups = [1,1,2,2,2]
        >>> l1weights = [1,1,1,1,1]
        >>> penalty = rr.sparse_group_lasso(groups, l1weights, lagrange=1.)
        >>> arg = [2,4,5,3,4]
        >>> list(penalty.terms(arg)) # doctest: +ELLIPSIS
        [12.3245..., 24.2474...]
        >>> penalty.seminorm(arg) # doctest: +ELLIPSIS
        36.5720...
        >>> np.sqrt((2**2 + 4**2)*2) + 6, np.sqrt((5**2 + 3**2 + 4**2) * 3.) + 12 # doctest: +ELLIPSIS
        (12.3245..., 24.2474...)
        >>> np.sqrt((2**2 + 4**2)*2) + np.sqrt((5**2 + 3**2 + 4**2) * 3.) + np.sum(np.fabs(arg)) # doctest: +ELLIPSIS
        36.5720...
        
        """
        arg = np.asarray(arg)
        norms = []
        for g in np.unique(self.groups):
            group = self.groups == g
            arg_g = arg[group]
            term = np.linalg.norm(arg_g) * self.weights[g]
            term += np.fabs(arg_g * self.lasso_weights[group]).sum()
            norms.append(term)
        return np.array(norms)

    def __repr__(self):
        if self.lagrange is not None:
            if not self.quadratic.iszero:
                return "%s(%s, %s, weights=%s, lagrange=%f, offset=%s)" % \
                    (self.__class__.__name__,
                     str(self.groups),
                     str(self.lasso_weights),
                     str(self.weights),
                     self.lagrange,
                     str(self.offset))
            else:
                return "%s(%s, %s, weights=%s, lagrange=%f, offset=%s, quadratic=%s)" % \
                    (self.__class__.__name__, 
                     str(self.groups),
                     str(self.lasso_weights),
                     str(self.weights),
                     self.lagrange,
                     str(self.offset),
                     self.quadratic)
        else:
            if not self.quadratic.iszero:
                return "%s(%s, %s, weights=%s, bound=%f, offset=%s)" % \
                    (self.__class__.__name__,
                     str(self.groups),
                     str(self.lasso_weights),
                     str(self.weights),
                     self.bound,
                     str(self.offset))
            else:
                return "%s(%s, %s, weights=%s, bound=%f, offset=%s, quadratic=%s)" % \
                    (self.__class__.__name__,
                     str(self.groups),
                     str(self.lasso_weights),
                     str(self.weights),
                     self.bound,
                     str(self.offset),
                     self.quadratic)

    @doc_template_user
    def get_conjugate(self):
        if self.quadratic.coef == 0:

            offset, outq = _work_out_conjugate(self.offset, self.quadratic)

            if self.bound is None:
                cls = conjugate_sparse_group_lasso_pairs[self.__class__]
                atom = cls(self.groups,
                           self.lasso_weights,
                           weights=self.weights, 
                           bound=self.lagrange, 
                           lagrange=None,
                           offset=offset,
                           quadratic=outq)
            else:
                cls = conjugate_sparse_group_lasso_pairs[self.__class__]
                atom = cls(self.groups,
                           self.lasso_weights,
                           weights=self.weights, 
                           lagrange=self.bound, 
                           bound=None,
                           offset=offset,
                           quadratic=outq)
        else:
            atom = smooth_conjugate(self)
            
        self._conjugate = atom
        self._conjugate._conjugate = self
        return self._conjugate
    conjugate = property(get_conjugate)

@objective_doc_templater()
class sparse_group_lasso_dual(sparse_group_lasso):

     r"""
     The dual of the slope penalty:math:`\ell_{\infty}` norm
     """

     objective_template = r"""{\cal D}^{group sparse}(%(var)s)"""
     objective_vars = seminorm.objective_vars.copy()

     @property
     def base_conjugate(self):
         """
         For a sparse group LASSO dual in bound form,
         an atom that agrees with its primal Lagrange form.
         
         Note: this conjugate does not take into account the atom's quadratic
         and offset terms. 
         """
         if not hasattr(self, "_conjugate_atom"):
              self._base_conjugate_atom = sparse_group_lasso(  
                                             self.groups,
                                             self.lasso_weights,
                                             weights=self.weights,
                                             lagrange=self.bound)
         return self._base_conjugate_atom

     @doc_template_user
     def seminorm(self, x, lagrange=None, check_feasibility=False):
         lagrange = seminorm.seminorm(self, x, 
                                      check_feasibility=check_feasibility, 
                                      lagrange=lagrange)
         return gauge_function_dual(self.base_conjugate, x) * lagrange

     @doc_template_user
     def constraint(self, x, bound=None):
         bound = seminorm.constraint(self, x, bound=bound)
         return inside_set(self.base_conjugate, x)

     @doc_template_user
     def lagrange_prox(self, x,  lipschitz=1, lagrange=None):
         raise NotImplementedError

     @doc_template_user
     def bound_prox(self, x, bound=None):
          bound = seminorm.bound_prox(self, x, bound)
          resid = self.base_conjugate.lagrange_prox(x, lagrange=bound)
          return x - resid

     def terms(self, arg):
         """
         Return the args that are maximized
         in computing the seminorm.
         
         >>> import regreg.api as rr
         >>> groups = [1,1,2,2,2]
         >>> l1weights = [1,1,1,1,1]
         >>> penalty = rr.sparse_group_lasso_dual(groups, l1weights, lagrange=1.)
         >>> arg = [2,4,5,3,4]
         >>> list(penalty.terms(arg)) # doctest: +ELLIPSIS
         [1.666..., 2.0833...]

         """
         arg = np.asarray(arg)
         norms = []
         for g in np.unique(self.groups):
             group = self.groups == g
             arg_g = arg[group]
             term = _gauge_function_dual_strong(arg_g,
                                                self.lasso_weights[group],
                                                self.weights[g])[0]
             norms.append(term)
         return np.array(norms)

def inside_set(atom, point):
    """
    Is the point in the dual ball?

    If the atom is the primal (necessarily in lagrange form), we check
    whether point is in the dual ball (i.e.
    the one determining the norm.

    If the atom is a dual (necessarily in bound form), we just check
    if we project onto the same point.
    """

    if atom.__class__ == sparse_group_lasso:
        proj_point = point - atom.lagrange_prox(point,
                                                lipschitz=1,
                                                lagrange=1)
    elif atom.__class__ == sparse_group_lasso_dual:
        proj_point = atom.bound_prox(point)
    else:
        raise ValueError('must be a sparse group lasso class')

    if np.linalg.norm(proj_point - point) > max(np.linalg.norm(point), 1) * 1.e-7:
        return False
    return True

def gauge_function_dual(atom,
                        point,
                        tol=1.e-10,
                        max_iter=50): 

     """
     Work out dual norm of sparse group LASSO by binary search.

     NOTE: will have problems if the atom has infinite feature weights
     """

     point = np.asarray(point)

     # find upper and lower bounds

     lower, upper = 1., 1.
     point_inside = inside_set(atom, point)
     
     iter = 0
     if point_inside:
          # gauge is upper bounded by 1
          # find a lower bound

          while True:
               lower = lower / 2
               candidate = point / lower

               if not inside_set(atom, candidate):
                    break
               else:
                    upper = lower

               iter += 1
               if iter == max_iter:
                    return 0
     else:
          # gauge is lower bounded by 1
          # find an upper bound

          while True:
               upper *= 2
               candidate = point / upper

               if inside_set(atom, candidate):
                    break
               else:
                    lower = upper

               iter += 1
               if iter == max_iter:
                    return np.inf

     # binary search

     assert (not inside_set(atom, point / lower))
     assert inside_set(atom, point / upper)

     while (upper - lower) > tol * 0.5 * (upper + lower):
         mid = 0.5 * (upper + lower)
         candidate = point / mid
         if inside_set(atom, candidate):
             upper = mid
         else:
             lower = mid
     return mid

# bookkeeping for automatically computing conjugates

conjugate_sparse_group_lasso_pairs = {}
for n1, n2 in [(sparse_group_lasso, sparse_group_lasso_dual)]:
    conjugate_sparse_group_lasso_pairs[n1] = n2
    conjugate_sparse_group_lasso_pairs[n2] = n1

# for terms of strong rules

def _inside_set_strong(point, bound, lasso_weights, group_weight, tol=1.e-5):

    # soft_thresh = np.sign(point) * np.maximum(np.fabs(point) - bound * lasso_weights, 0)
    # sign doesn't matter for testing inside the dual ball
    soft_thresh = np.maximum(np.fabs(point) - bound * lasso_weights, 0)
    norm_soft = np.linalg.norm(soft_thresh)
    if norm_soft <= bound * (group_weight + tol):
        return True
    return False

def _gauge_function_dual_strong(point,
                                lasso_weights, 
                                group_weight,
                                tol=1.e-6,
                                max_iter=50): 

     """
     Work out dual norm of sparse group LASSO by binary search.

     NOTE: will have problems if the atom has infinite feature weights
     """

     point = np.asarray(point)

     # find upper and lower bounds

     lower, upper = 1., 1.
     point_inside = _inside_set_strong(point,
                                       lower, 
                                       lasso_weights,
                                       group_weight)
     
     iter = 0
     if point_inside:
          # gauge is upper bounded by 1
          # find a lower bound

          while True:
               lower = lower / 2
               if not _inside_set_strong(point,
                                         lower,
                                         lasso_weights,
                                         group_weight):
                    break
               else:
                    upper = lower

               iter += 1
               if iter == max_iter:
                    return 0, None, None
     else:
          # gauge is lower bounded by 1
          # find an upper bound

          while True:
               upper *= 2
               if _inside_set_strong(point,
                                     upper, 
                                     lasso_weights,
                                     group_weight):
                    break
               else:
                    lower = upper

               iter += 1
               if iter == max_iter:
                    return np.inf, None, None

     # binary search

     assert (not _inside_set_strong(point,
                                    lower,
                                    lasso_weights,
                                    group_weight))
     assert _inside_set_strong(point,
                               upper,
                               lasso_weights,
                               group_weight)

     while (upper - lower) > tol * 0.5 * (upper + lower):
         mid = 0.5 * (upper + lower)
         if _inside_set_strong(point,
                               mid,
                               lasso_weights,
                               group_weight):
             upper = mid
         else:
             lower = mid

     soft_thresh = np.maximum(np.fabs(point) - mid * lasso_weights, 0) * np.sign(point)
     l1_subgrad = point - soft_thresh # a point in the appropriate cube
     l2_subgrad = soft_thresh / np.linalg.norm(soft_thresh) * group_weight * mid
     return mid, l1_subgrad, l2_subgrad




import numpy as np
try:
    from sklearn.model_selection import cross_validate
    from sklearn.model_selection import GridSearchCV
    from ..api import (huber_svm,
                       huber_svm_lagrange)
    have_sklearn = True
except ImportError:
    have_sklearn = False

from ...api import l1norm
from ...tests.decorators import set_seed_for_test

@np.testing.dec.skipif(not have_sklearn)
@set_seed_for_test()
def test_regression_huber_svm():

    n, p = 100, 20
    X = np.random.standard_normal((n, p))
    y = np.random.standard_normal(n)

    atom = l1norm
    atom_args = {'shape': p,
                 'lagrange': 2 * np.sqrt(n)}
    huber_svm_lasso = huber_svm(0.5, atom, atom_args)
    huber_svm_lasso.fit(X, y)
    print(cross_validate(huber_svm_lasso, X, y, cv=10))

    # grid search
    params = {'atom_params':[{'shape':p,
                              'lagrange': alpha * 2 * np.sqrt(n)} for alpha in [0.5, 1, 1.5]]}
    lasso_cv = GridSearchCV(huber_svm_lasso, params, cv=3)
    lasso_cv.fit(X, y)

    huber_svm_lasso_offset = huber_svm(0.5, atom, atom_args,
                                       offset=True)
    y_offset = np.array([y, np.random.standard_normal(n)]).T
    huber_svm_lasso_offset.fit(X, y_offset)

    huber_svm_lasso_weights = huber_svm(0.5, atom, atom_args,
                                        case_weights=True,
                                        score_method='mean_deviance')
    
    y_weights = np.array([y, np.ones_like(y)]).T
    huber_svm_lasso_weights.fit(X, y_weights)

    np.testing.assert_allclose(huber_svm_lasso_weights._coefs,
                               huber_svm_lasso._coefs)

    GridSearchCV(huber_svm_lasso_offset, params, cv=3).fit(X, y_offset)
    GridSearchCV(huber_svm_lasso_weights, params, cv=3).fit(X, y_weights)

    huber_svm_lasso_both = huber_svm(0.5, atom, atom_args,
                                     offset=True,
                                     case_weights=True,
                                     score_method='R2')
    y_both = np.array([y, np.ones_like(y), y_offset[:,1]]).T
    GridSearchCV(huber_svm_lasso_both, params, cv=3).fit(X, y_both)

    def atom_constructor(null_grad, **args):
        return l1norm(**args)
    huber_svm_lasso_enet = huber_svm_lagrange(0.5,
                                              atom_constructor,
                                              atom_args,
                                              offset=True,
                                              enet_alpha=0.5,
                                              case_weights=True,
                                              score_method='R2')
    y_both = np.array([y, np.ones_like(y), y_offset[:,1]]).T
    GridSearchCV(huber_svm_lasso_enet, params, cv=3).fit(X, y_both)






from copy import copy

import numpy as np
from ..affine import affine_transform

class multiscale(affine_transform):

    """
    An affine transform representing the
    multiscale changepoint transform.

    This transform centers the input
    and then computes the average over
    all intervals of a minimum size.
    """

    dtype = np.dtype([('start', np.int),
                      ('end', np.int)])

    def __init__(self, p, minsize=None, slices=[],
                 scaling=None):
        """
        Parameters
        ----------

        p : int
            Length of signal.

        minsize : int
            Smallest interval to consider.
            Defaults to p**(1/3.)

        slices : []
            A list of intervals to use in the transform.
            Will be coerced to have dtype([('start', np.int), ('end', np.int)])

        scaling : np.float((slices.shape))
            An optional scaling to apply after computing
            mean over each interval.

        """
        self.p = p
        self.minsize = minsize or int(np.around(p**(1/3.)))
        self.slices = np.asarray(slices, self.dtype)
        self.scaling = scaling
        if self.slices.shape in [(), (0,)]:
            self._all = True # if computing all differences of a certain size things can be sped up
            S = np.indices((p,p)).reshape((2,-1))
            self._mask = (S[1]-S[0] >= self.minsize)
            slices = S[:,self._mask].T
            self.slices = np.empty(slices.shape[0], self.dtype)
            self.slices['start'] = slices[:,0]
            self.slices['end'] = slices[:,1]
        else:
            self._all = False
        self.sizes = self.slices['end'] - self.slices['start'] * 1.
        self.input_shape = (p,)
        self.output_shape = (len(self.slices),)

    def __copy__(self):
        M = multiscale(self.p)
        M.minsize = self.minsize
        M.slices = copy(self.slices)
        M.scaling = copy(self.scaling)
        if hasattr(self, "_mask"):
            M._mask = copy(self._mask)
        M._all = self._all
        M.sizes = copy(self.sizes)
        M.input_shape = self.input_shape
        M.output_shape = self.output_shape
        return M

    def update_slices(self, slices, scaling=None):
        """

        Change the intervals computed by the multiscale
        transform.

        Parameters
        ----------

        slices : []
            List of (i,j) intervals to compute.
            Will be converted to have dtype([('start', np.int), ('end', np.int)])

        """
        slices = np.asarray(slices)
        if slices.dtype != self.dtype:
            self.slices = np.empty(slices.shape[0], self.dtype)
            self.slices['start'] = slices[:,0]
            self.slices['end'] = slices[:,1]
            self._all = False
        else:
            self.slices = slices
            self._all = False
        self.sizes = self.slices['end'] - self.slices['start'] * 1.
        self.output_shape = (len(self.slices),)
        self.scaling = scaling

    def linear_map(self, x):
        """
        Given a p-vector `x` compute the average of
        `x - x.mean()` over each interval
        of size greater than `self.minsize`.

        Parameters
        ----------

        x : np.float(self.input_shape)

        Returns
        -------

        v : np.float(self.output_shape)

        """
        x_centered = x - x.mean()
        output = np.zeros(self.output_shape)
        cumsum = np.cumsum(x_centered)
        if not self._all:
            for k, ij in enumerate(self.slices):
                i, j = ij
                if i >= 1:
                    output[k] = cumsum[j-1] - cumsum[i-1] 
                else:
                    output[k] = cumsum[j-1]
                output[k] /= j - i
        else:
            _cumsum = np.hstack([np.zeros(1), cumsum])
            D = np.subtract.outer(_cumsum, _cumsum)[:-1,:-1].reshape(-1)[self._mask]
            output = -D / self.sizes
        if self.scaling is not None:
            output *= self.scaling
        return output

    def affine_map(self, x):
        return self.linear_map(x)

    def adjoint_map(self, v):
        """
        Parameters
        ----------

        v : np.float(self.output_shape)

        Returns
        -------

        v : np.float(self.input_shape)
        """

        if v.ndim == 1:
            v_scaled = v / self.sizes
        else:
            v_scaled = v / self.sizes[:,None]
        if self.scaling is not None:
            v_scaled *= self.scaling
        if not self._all:
            output = np.zeros(self.input_shape)
            non0 = np.nonzero(v_scaled)[0]
            if non0.shape != ():
                for k in non0:
                    i, j = self.slices[k]
                    output[i:j] += v_scaled[k]
        else:
            _output1 = np.zeros((self.input_shape[0], self.input_shape[0])).reshape(-1)
            _output1[self._mask] = v_scaled
            _output1.shape = (self.input_shape[0], self.input_shape[0])
            _output2 = _output1.sum(0) - _output1.sum(1)
            output = -np.cumsum(_output2)
        return output - output.mean()
        
    def form_matrix(self, slices):
        """
        Form a matrix with given slices of multiscale transform.
        """
        p = self.input_shape[0]
        X_E = np.zeros((p, len(slices)))
        length = np.zeros(len(slices))
        for k, ij in enumerate(slices):
            i,j = ij
            X_E[i:j, k] = 1.
            length[k] = j-i

        X_hat = X_E - X_E.mean(0)[None,:]
        return X_hat / np.sqrt(length), length

def SMUCE_stat(M, Z, sigma=None):
    """

    Compute multiscale test statistic of Chan and Walther. Used
    in the SMUCE algorithm and in the multiscale change point
    algorithm in the square-root LASSO selective inference paper.

    Parameters
    ----------

    M : instance of `multiscale`
    
    Z : np.ndarray
        Data on which to compute the statistic.

    sigma : float
        Noise level. Defaults to np.std(Z).

    Notes
    -----

    M's scaling is ignored, even if not None.

    """

    p = Z.shape[0]
    M.scaling, old = np.sqrt(M.sizes), M.scaling
    Z0 = Z - Z.mean()
    if sigma is None:
        sigma_hat = np.std(Z)
    T = np.fabs(M.linear_map(Z0) / sigma_hat) - np.sqrt(2 * np.log(p / M.sizes))
    idx = np.argmax(T)
    return T[idx], M.slices[idx]

def choose_tuning_parameter(M, ndraw=100, quantile=0.95, sigma=None):
    """

    Choose tuning parameter 
    in the SMUCE algorithm and in the multiscale change point
    algorithm in the square-root LASSO selective inference paper.

    Parameters
    ----------

    M : instance of `multiscale`
    
    ndraw : int
        How many Monte Carlo draws.

    quantile : float
        Which quantile of Monte Carlo draws to choose.

    sigma : float
        Passed to `SMUCE_stat`

    Notes
    -----

    M's scaling is ignored, even if not None.

    """
    V = []
    p = M.input_shape[0]
    for i in range(ndraw):
        Z = np.random.standard_normal(p)
        V.append(SMUCE_stat(M, Z)[0])
    return np.percentile(V, 100*quantile)




import numpy as np

from .. import lasso, sparse_group_block, strong_rules, warm_start
from ..basil import basil_inner_loop, basil
from ...tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_gaussian_multiresponse():
    '''
    run a basic path algorithm

    '''
    n, p, q = 200, 50, 3
    X = np.random.standard_normal((n,p))
    betaX = np.zeros((p,q))
    betaX[:3] = [3,4,5]
    np.random.shuffle(betaX)
    Y = np.dot(X, betaX) + np.random.standard_normal((n,q))

    sparse_group_block1 = sparse_group_block.multiresponse_gaussian(X, 
                                                                    Y, 
                                                                    1,
                                                                    np.sqrt(q))
    lagrange_sequence = sparse_group_block.default_lagrange_sequence(sparse_group_block1.penalty,
                                                                     sparse_group_block1.grad_solution,
                                                                     nstep=23) # initialized at "null" model
    sol1 = strong_rules(sparse_group_block1, 
                        lagrange_sequence, 
                        (sparse_group_block1.solution, sparse_group_block1.grad_solution),
                        inner_tol=1.e-12)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_gaussian_blocks():
    '''
    run a basic path algorithm

    '''
    n1, n2, n3, p, q = 190, 200, 210, 50, 3
    X1 = np.random.standard_normal((n1,p))
    X2 = np.random.standard_normal((n2,p))
    X3 = np.random.standard_normal((n3,p))
    betaX = np.zeros((p,q))
    betaX[:3] = [3,4,5]
    np.random.shuffle(betaX)
    Y = [np.dot(X, beta) + np.random.standard_normal(n) for
         X, beta, n in zip([X1, X2, X3],
                           betaX.T,
                           [n1, n2, n3])]
                                     

    sparse_group_block1 = sparse_group_block.stacked_gaussian([X1, X2, X3], 
                                                              Y, 
                                                              1,
                                                              np.sqrt(q))
    lagrange_sequence = sparse_group_block.default_lagrange_sequence(sparse_group_block1.penalty,
                                                                     sparse_group_block1.grad_solution,
                                                                     nstep=23) # initialized at "null" model
    sol1 = strong_rules(sparse_group_block1, 
                        lagrange_sequence, 
                        (sparse_group_block1.solution, sparse_group_block1.grad_solution),
                        inner_tol=1.e-12)
    beta1 = sol1['beta']

    print(np.sqrt((beta1**2).sum(2).sum(1)))
    
@set_seed_for_test()
def test_multinomial():
    '''
    run a basic path algorithm with multinomial loss

    '''
    n, p, q = 200, 50, 3
    X = np.random.standard_normal((n,p))
    betaX = np.zeros((p,q))
    betaX[:3] = np.array([3,4,5]) / np.sqrt(n)
    np.random.shuffle(betaX)
    eta = np.dot(X, betaX)
    prob = np.exp(eta) / (1 + np.exp(eta))
    Y = np.random.binomial(1, prob) # not really "multinomial", but it will do for testing

    sparse_group_block1 = sparse_group_block.multinomial(X, 
                                                         Y, 
                                                         1,
                                                         np.sqrt(q))
    lagrange_sequence = sparse_group_block.default_lagrange_sequence(sparse_group_block1.penalty,
                                                                     sparse_group_block1.grad_solution,
                                                                     nstep=23) # initialized at "null" model
    sol1 = strong_rules(sparse_group_block1, 
                        lagrange_sequence, 
                        (sparse_group_block1.solution, sparse_group_block1.grad_solution),
                        inner_tol=1.e-12)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_path_subsample(n=200,p=50):
    '''
    compare a subsample path to the full path on subsampled data

    '''
    n, p, q = 200, 50, 3
    X = np.random.standard_normal((n,p))
    betaX = np.zeros((p,q))
    betaX[:3] = [3,4,5]
    np.random.shuffle(betaX)
    Y = np.dot(X, betaX) + np.random.standard_normal((n,q))

    sparse_group_block1 = sparse_group_block.multiresponse_gaussian(X, 
                                                                    Y, 
                                                                    1,
                                                                    np.sqrt(q))
    cases = range(100)

    sparse_group_block1 = sparse_group_block1.subsample(cases)
    lagrange_sequence = sparse_group_block.default_lagrange_sequence(sparse_group_block1.penalty,
                                                                     sparse_group_block1.grad_solution,
                                                                     nstep=23) # initialized at "null" model
    sol1 = strong_rules(sparse_group_block1, 
                        lagrange_sequence, 
                        (sparse_group_block1.solution, sparse_group_block1.grad_solution),
                        inner_tol=1.e-10)
    beta1 = sol1['beta']

    sparse_group_block2 = sparse_group_block.multiresponse_gaussian(X[cases], 
                                                                    Y[cases], 
                                                                    1,
                                                                    np.sqrt(q))
    sol2 = strong_rules(sparse_group_block2, 
                        lagrange_sequence, 
                        (sparse_group_block2.solution, sparse_group_block2.grad_solution),
                        inner_tol=1.e-10)
    beta2 = sol2['beta']

    np.testing.assert_allclose(beta1, beta2, rtol=1.e-3)

@set_seed_for_test()
def test_elastic_net(n=200, p=50):
    '''
    run a basic elastic net path algorithm

    '''
    n, p, q = 200, 50, 3
    X = np.random.standard_normal((n,p))
    betaX = np.zeros((p,q))
    betaX[:3] = [3,4,5]
    np.random.shuffle(betaX)
    Y = np.dot(X, betaX) + np.random.standard_normal((n,q))


    enet = np.zeros((X.shape[1], Y.shape[1]))
    enet[4:7] = 0

    sparse_group_block1 = sparse_group_block.multiresponse_gaussian(X, 
                                                                    Y, 
                                                                    1,
                                                                    np.sqrt(2),
                                                                    alpha=0.5,
                                                                    elastic_net_param=enet)
    lagrange_sequence = sparse_group_block.default_lagrange_sequence(sparse_group_block1.penalty,
                                                                     sparse_group_block1.grad_solution,
                                                                     nstep=23) # initialized at "null" model
    sol1 = strong_rules(sparse_group_block1, 
                        lagrange_sequence, 
                        (sparse_group_block1.solution, sparse_group_block1.grad_solution),
                        inner_tol=1.e-12)
    beta1 = sol1['beta']


@set_seed_for_test()
def test_basil_inner_loop(n=200,p=50):
    '''
    test one run of the BASIL inner loop

    '''
    n, p, q = 200, 50, 3
    X = np.random.standard_normal((n,p))
    betaX = np.zeros((p,q))
    betaX[:3] = [3,4,5]
    np.random.shuffle(betaX)
    Y = np.dot(X, betaX) + np.random.standard_normal((n,q))

    enet = np.zeros((X.shape[1], Y.shape[1]))
    enet[4:7] = 0

    sparse_group_block1 = sparse_group_block.multiresponse_gaussian(X, 
                                                                    Y, 
                                                                    1,
                                                                    np.sqrt(2),
                                                                    alpha=0.5,
                                                                    elastic_net_param=enet)
    lagrange_sequence = sparse_group_block.default_lagrange_sequence(sparse_group_block1.penalty,
                                                              sparse_group_block1.grad_solution,
                                                              nstep=100) # initialized at "null" model
    sol1 = basil_inner_loop(sparse_group_block1, 
                            lagrange_sequence[:50], 
                            (sparse_group_block1.solution.copy(), sparse_group_block1.grad_solution.copy()),
                            inner_tol=1.e-14,
                            step_nvar=10)
    lagrange1, beta1, grad1, active1 = sol1
    print(np.array(beta1).shape, 'chunk of path')

    print(active1, 'active')

@set_seed_for_test()
def test_basil(n=300,p=100):
    '''
    run BASIL

    '''
    n, p, q = 200, 50, 3
    X = np.random.standard_normal((n,p))
    betaX = np.zeros((p,q))
    betaX[:3] = [3,4,5]
    np.random.shuffle(betaX)
    Y = np.dot(X, betaX) + np.random.standard_normal((n,q))

    sparse_group_block1 = sparse_group_block.multiresponse_gaussian(X, 
                                                                    Y, 
                                                                    1,
                                                                    np.sqrt(2))
    lagrange_sequence = sparse_group_block.default_lagrange_sequence(sparse_group_block1.penalty,
                                                                     sparse_group_block1.grad_solution,
                                                                     lagrange_proportion=0.4,
                                                                     nstep=50) # initialized at "null" model
    sol1 = basil(sparse_group_block1, 
                 lagrange_sequence, 
                 (sparse_group_block1.solution.copy(), sparse_group_block1.grad_solution.copy()),
                 inner_tol=1.e-14,
                 step_nvar=10,
                 step_lagrange=20)
    
    sparse_group_block2 = sparse_group_block.multiresponse_gaussian(X, 
                                                                    Y, 
                                                                    1,
                                                                    np.sqrt(2))

    return sparse_group_block2



from copy import copy
import warnings

import numpy as np

from ..problems.composite import composite, nonsmooth, smooth_conjugate
from ..affine import linear_transform, identity as identity_transform, selector
from ..identity_quadratic import identity_quadratic
from ..atoms import _work_out_conjugate, atom
from ..smooth import affine_smooth

from ..objdoctemplates import objective_doc_templater
from ..doctemplates import (doc_template_user, doc_template_provider)

# Constants used below

UNPENALIZED = -1
L1_PENALTY = -2
POSITIVE_PART = -3
NONNEGATIVE = -4

reserved = [UNPENALIZED, L1_PENALTY, POSITIVE_PART,
            NONNEGATIVE]

try:
    from .mixed_lasso_cython import (mixed_lasso_lagrange_prox, 
                                     mixed_lasso_dual_bound_prox,
                                     mixed_lasso_bound_prox,
                                     seminorm_mixed_lasso,
                                     seminorm_mixed_lasso_dual,
                                     strong_set_mixed_lasso,
                                     mixed_lasso_epigraph,
                                     check_KKT_mixed_lasso)
except ImportError:
    raise ImportError('need cython module mixed_lasso_cython')

@objective_doc_templater()
class mixed_lasso(atom):

    objective_template = 'needs a template'

    """
    A class that defines the API for cone constraints.
    """
    tol = 1.0e-05

    def __init__(self, 
                 penalty_structure, 
                 lagrange, 
                 weights={},
                 offset=None,
                 quadratic=None,
                 initial=None):
        shape = np.asarray(penalty_structure).shape
        nonsmooth.__init__(self, shape, offset,
                           quadratic, initial)

        self.weights = weights
        self.lagrange = lagrange
        self.penalty_structure = np.asarray(penalty_structure)
        self._groups = -np.ones(self.shape, np.intp)
        groups = set(np.unique(self.penalty_structure)).difference(
            set(reserved))
        self._weight_array = np.zeros(len(groups))

        self._l1_penalty = np.nonzero(self.penalty_structure == L1_PENALTY)[0].astype(np.intp)
        self._positive_part = np.nonzero(self.penalty_structure == POSITIVE_PART)[0].astype(np.intp) 
        self._unpenalized = np.nonzero(self.penalty_structure == UNPENALIZED)[0].astype(np.intp) 
        self._nonnegative = np.nonzero(self.penalty_structure == NONNEGATIVE)[0].astype(np.intp)

        for idx, label in enumerate(groups):
            g = self.penalty_structure == label
            self._groups[g] = idx
            self._weight_array[idx] = self.weights.get(label, np.sqrt(g.sum()))

        # buffers for cython computations

        self._norms_cython = np.zeros_like(self._weight_array)
        self._factors_cython = np.zeros_like(self._weight_array)
        self._projection_cython = np.zeros(self.shape)

    def __eq__(self, other):
        if self.__class__ == other.__class__:
            return (self.shape == other.shape and 
                    np.all(self.penalty_structure == other.penalty_structure)
                    and np.all(self.weights == other.weights)
                    and self.lagrange == other.lagrange)
        return False

    def __copy__(self):
        return self.__class__(copy(self.penalty_structure),
                              self.lagrange,
                              weights=self.weights,
                              offset=copy(self.offset),
                              initial=self.coefs,
                              quadratic=self.quadratic)
    
    def __repr__(self):
        if self.quadratic.iszero:
            return "%s(%r, %s, weights=%r, offset=%s)" % \
                (self.__class__.__name__,
                 self.penalty_structure,
                 self.lagrange,
                 self.weights,
                 str(self.offset))
        else:
            return "%s(%s, %r, weights=%r, offset=%s, quadratic=%s)" % \
                (self.__class__.__name__,
                 self.penalty_structure,
                 self.lagrange,
                 self.weights,
                 str(self.offset),
                 str(self.quadratic))

    @property
    def conjugate(self):
        if self.quadratic.coef == 0:
            offset, outq = _work_out_conjugate(self.offset, 
                                               self.quadratic)
            cls = mixed_lasso_dual
            atom = cls(self.penalty_structure,
                       self.lagrange,
                       weights=self.weights,
                       offset=offset,
                       quadratic=outq)
        else:
            atom = smooth_conjugate(self)
        self._conjugate = atom
        self._conjugate._conjugate = self
        return self._conjugate

    @property
    def dual(self):
        return self.linear_transform, self.conjugate

    @property
    def linear_transform(self):
        if not hasattr(self, "_linear_transform"):
            self._linear_transform = identity_transform(self.shape)
        return self._linear_transform
    
    @doc_template_provider
    def constraint(self, x, bound=None):
        r"""
        Verify :math:`\cdot %(objective)s \leq \lambda`, where :math:`\lambda`
        is bound, :math:`\alpha` is self.offset (if any).

        If True, returns 0, else returns np.inf.

        The class atom's constraint just returns the appropriate bound
        parameter for use by the subclasses.
        """
        if bound is None:
            raise ValueError('bound must be suppled')
        x_offset = self.apply_offset(x)
        return self.seminorm(x_offset) <= bound

    def nonsmooth_objective(self, x, check_feasibility=False):
        x_offset = self.apply_offset(x)
        v = self.seminorm(x_offset, check_feasibility=check_feasibility)
        v += self.quadratic.objective(x, 'func')
        return v

    def seminorm(self, x, check_feasibility=False):
        x_offset = self.apply_offset(x)
        # flush buffers for computations

        self._norms_cython[:] = 0

        v = seminorm_mixed_lasso(x_offset,
                                 self._l1_penalty,
                                 self._unpenalized,
                                 self._positive_part,
                                 self._nonnegative,
                                 self._groups,
                                 self._weight_array,
                                 self._norms_cython,
                                 np.int(check_feasibility))

        return v * self.lagrange

    def proximal(self, proxq, prox_control=None):
        r"""
        The proximal operator. If the atom is in
        Lagrange mode, this has the form

        .. math::

           v^{\lambda}(x) = \text{argmin}_{v \in \mathbb{R}^p} \frac{L}{2}
           \|x-v\|^2_2 + \lambda h(v+\alpha) + \langle v, \eta \rangle

        where :math:`\alpha` is the offset of self.affine_transform and
        :math:`\eta` is self.linear_term.

        .. math::

           v^{\lambda}(x) = \text{argmin}_{v \in \mathbb{R}^p} \frac{L}{2}
           \|x-v\|^2_2 + \langle v, \eta \rangle \text{s.t.} \   h(v+\alpha) \leq \lambda

        """

        offset, totalq = (self.quadratic + proxq).recenter(self.offset)
        if totalq.coef == 0:
            raise ValueError('lipschitz + quadratic coef must be positive')

        prox_arg = -totalq.linear_term / totalq.coef

        # flush buffers for computations

        self._norms_cython[:] = 0
        self._factors_cython[:] = 0
        self._projection_cython[:] = 0

        eta = mixed_lasso_lagrange_prox(prox_arg, 
                                        self.lagrange, 
                                        totalq.coef, 
                                        self._l1_penalty,
                                        self._unpenalized,
                                        self._positive_part,
                                        self._nonnegative,
                                        self._groups,
                                        self._weight_array,
                                        self._norms_cython,
                                        self._factors_cython,
                                        self._projection_cython)

        if offset is None:
            return eta
        else:
            return eta + offset


@objective_doc_templater()
class mixed_lasso_dual(mixed_lasso):

    _doc_dict = {'linear':r' + \langle \eta, x \rangle',
                 'constant':r' + \tau',
                 'objective': '',
                 'shape':'p',
                 'var':r'x'}

    """
    Conjugate of the group lasso seminorm (in bound form only for now).
    """
    tol = 1.0e-05

    def __init__(self, 
                 penalty_structure, 
                 bound, 
                 weights={},
                 offset=None,
                 quadratic=None,
                 initial=None):

        mixed_lasso.__init__(self, 
                             penalty_structure, 
                             bound, 
                             weights=weights,
                             offset=offset,
                             quadratic=quadratic,
                             initial=initial)
        del(self.lagrange)
        self.bound = bound

    def __eq__(self, other):
        if self.__class__ == other.__class__:
            return (self.shape == other.shape and 
                    np.all(self.penalty_structure == other.penalty_structure)
                    and np.all(self.weights == other.weights)
                    and self.bound == other.bound)
        return False

    def __copy__(self):
        return self.__class__(copy(self.penalty_structure),
                              self.bound,
                              weights=self.weights,
                              offset=copy(self.offset),
                              initial=self.coefs,
                              quadratic=self.quadratic)
    
    def __repr__(self):
        if self.quadratic.iszero:
            return "%s(%s, %r, weights=%r, offset=%s)" % \
                (self.__class__.__name__,
                 self.bound,
                 self.penalty_structure,
                 self.weights,
                 str(self.offset))
        else:
            return "%s(%s, %r, weights=%r, offset=%s, quadratic=%s)" % \
                (self.__class__.__name__,
                 self.bound, 
                 self.penalty_structure,
                 self.weights,
                 str(self.offset),
                 str(self.quadratic))

    @property
    def conjugate(self):
        if self.quadratic.coef == 0:
            offset, outq = _work_out_conjugate(self.offset, 
                                               self.quadratic)
            cls = mixed_lasso
            atom = cls(self.penalty_structure,
                       self.bound,
                       weights=self.weights,
                       offset=offset,
                       quadratic=outq)
        else:
            atom = smooth_conjugate(self)
        self._conjugate = atom
        self._conjugate._conjugate = self
        return self._conjugate

    @property
    def dual(self):
        return self.linear_transform, self.conjugate

    @property
    def linear_transform(self):
        if not hasattr(self, "_linear_transform"):
            self._linear_transform = identity_transform(self.shape)
        return self._linear_transform
    
    @doc_template_user
    def constraint(self, x, bound=None):
        if bound is None:
            raise ValueError('bound must be suppled')
        x_offset = self.apply_offset(x)
        return self.seminorm(x_offset) <= bound

    def nonsmooth_objective(self, x, check_feasibility=False):
        x_offset = self.apply_offset(x)
        if check_feasibility:
            v = self.constraint(x_offset, self.bound)
        else:
            v = 0
        v += self.quadratic.objective(x, 'func')
        return v

    def seminorm(self, x, lagrange=1, check_feasibility=False):
        x_offset = self.apply_offset(x)

        # flush buffers for computations

        self._norms_cython[:] = 0

        v = seminorm_mixed_lasso_dual(x_offset, 
                                      self._l1_penalty,
                                      self._unpenalized,
                                      self._positive_part,
                                      self._nonnegative,
                                      self._groups,
                                      self._weight_array,
                                      self._norms_cython)
        return v 

    def proximal(self, proxq, prox_control=None):
        r"""
        The proximal operator. If the atom is in
        Bound mode, this has the form

        .. math::

           v^{\lambda}(x) = \text{argmin}_{v \in \mathbb{R}^p} \frac{L}{2}
           \|x-v\|^2_2 + \lambda h(v+\alpha) + \langle v, \eta \rangle

        where :math:`\alpha` is the offset of self.affine_transform and
        :math:`\eta` is self.linear_term.

        .. math::

           v^{\lambda}(x) = \text{argmin}_{v \in \mathbb{R}^p} \frac{L}{2}
           \|x-v\|^2_2 + \langle v, \eta \rangle \text{s.t.} \   h(v+\alpha) \leq \lambda

        """
        offset, totalq = (self.quadratic + proxq).recenter(self.offset)
        if totalq.coef == 0:
            raise ValueError('lipschitz + quadratic coef must be positive')

        prox_arg = -totalq.linear_term / totalq.coef

        # flush buffers for computations

        self._norms_cython[:] = 0
        self._factors_cython[:] = 0
        self._projection_cython[:] = 0

        eta = mixed_lasso_dual_bound_prox(prox_arg, 
                                          self.bound, 
                                          self._l1_penalty,
                                          self._unpenalized,
                                          self._positive_part,
                                          self._nonnegative,
                                          self._groups,
                                          self._weight_array,
                                          self._norms_cython,
                                          self._factors_cython,
                                          self._projection_cython)

        if offset is None:
            return eta
        else:
            return eta + offset

def strong_set(glasso, 
               lagrange_cur, 
               lagrange_new, 
               grad,
               slope_estimate=1):

    """
    Guess at active variables at 
    lagrange value lagrange_new based on gradient
    at lagrange_cur.
    """

    p = grad.shape[0]
    value = strong_set_mixed_lasso(grad, 
                                   lagrange_new,
                                   lagrange_cur,
                                   slope_estimate,
                                   glasso._l1_penalty, 
                                   glasso._unpenalized,
                                   glasso._positive_part,
                                   glasso._nonnegative,
                                   glasso._groups,
                                   glasso._weight_array)
    value = value.astype(np.bool)
    return value, selector(value, (p,))

def check_KKT(glasso, 
              grad, 
              solution, 
              lagrange, 
              tol=1.e-2):

    """
    Check whether (grad, solution) satisfy
    KKT conditions at a given tolerance.
    """

    failing = check_KKT_mixed_lasso(grad, 
                                    solution, 
                                    lagrange,
                                    glasso._l1_penalty, 
                                    glasso._unpenalized,
                                    glasso._positive_part, 
                                    glasso._nonnegative,
                                    glasso._groups,
                                    glasso._weight_array,
                                    tol=tol)
    return failing > 0

conjugate_mixed_lasso_pairs = {}
for n1, n2 in [(mixed_lasso, mixed_lasso_dual)]:
    conjugate_mixed_lasso_pairs[n1] = n2
    conjugate_mixed_lasso_pairs[n2] = n1





import numpy as np
import warnings
try:
    from scipy.linalg import cho_factor, cho_solve, cholesky_banded, cho_solve_banded
except ImportError:
    warnings.warn('cannot import some cholesky solvers from scipy')

from ..affine import affine_transform, astransform, composition, scaler
from ..smooth import smooth_atom
from ..problems.composite import smooth_conjugate
from ..atoms.cones import zero
from ..atoms import _work_out_conjugate
from ..identity_quadratic import identity_quadratic

class quadratic_loss(smooth_atom):
    """
    Half of the square of the l2 norm

    Parameters
    ----------

    shape : tuple
       Shape of argument to `smooth_objective`

    Q: ndarray 
       positive definite matrix (optional),
       defaults to identity. If `Qdiag` then
       `Q` is one-dimensional.

    Qdiag : bool
       Is the quadratic form diagonal?

    coef : float (optional)
       Scalar multiple to be applied (must be nonnegative)

    offset : ndarray (optional)
       Vector to be subtracted before evaluating `smooth_objective`. 

    quadratic : `identity_quadratic` (optional)
       Instance of `identity_quadratic` to be added to overall
       objective.

    initial : ndarray (optional)
       Initial value for coefficients.

    """

    objective_vars = smooth_atom.objective_vars.copy()
    objective_vars['Q'] = 'Q'
    objective_template = r"""\frac{%(coef)s}{2} \cdot %(var)s^T %(Q)s %(var)s"""

    def __init__(self, 
                 shape, 
                 Q=None, 
                 Qdiag=False,
                 coef=1., 
                 offset=None,
                 quadratic=None,
                 initial=None):
        smooth_atom.__init__(self,
                             shape,
                             coef=coef,
                             offset=offset,
                             quadratic=quadratic,
                             initial=initial)
        
        self.Q = Q
        self.Qdiag = Qdiag
        if self.Q is not None:
            if Qdiag:
                self.Q_transform = scaler(Q)
            else:
                self.Q_transform = affine_transform(Q, None, False)
    @staticmethod
    def fromarray(Q, 
                  offset=None,
                  quadratic=None,
                  initial=None):
        return quadratic_loss((Q.shape[0],), 
                              Q=Q,
                              offset=offset,
                              quadratic=quadratic,
                              initial=initial)

    @staticmethod
    def squared_transform(transform, 
                          offset=None,
                          quadratic=None,
                          initial=None):
        transform = astransform(transform)
        Q = composition(transform.T, transform)
        return quadratic_loss(Q.input_shape, 
                              Q=Q,
                              offset=offset,
                              quadratic=quadratic,
                              initial=initial)

    @staticmethod
    def diagonal(D, 
                 offset=None,
                 quadratic=None,
                 initial=None):
        D = np.asarray(D)
        return quadratic_loss((D.shape[0],), 
                              Q=D,
                              Qdiag=True,
                              offset=offset,
                              quadratic=quadratic,
                              initial=initial)


    def __repr__(self):
        if self.quadratic.iszero:
            return "%s(%s, coef=%s, Q=%s, Qdiag=%s, offset=%s)" % \
                (self.__class__.__name__,
                 repr(self.shape), 
                 repr(self.coef),
                 repr(self.Q),
                 repr(self.Qdiag),
                 str(self.offset))
        else:
            return "%s(%s, coef=%s, Q=%s, Qdiag=%s, offset=%s, quadratic=%s)" % \
                (self.__class__.__name__,
                 repr(self.shape), 
                 repr(self.coef),
                 repr(self.Q),
                 repr(self.Qdiag),
                 str(self.offset),
                 self.quadratic)

    def smooth_objective(self, x, mode='both', check_feasibility=False):
        """
        Evaluate a smooth function and/or its gradient

        if mode == 'both', return both function value and gradient
        if mode == 'grad', return only the gradient
        if mode == 'func', return only the function value
        """

        x = self.apply_offset(x)
        if self.Q is None:
            if mode == 'both':
                f, g  = self.scale(np.linalg.norm(x)**2) / 2., self.scale(x)
                return f, g
            elif mode == 'grad':
                f, g = None, self.scale(x)
                return g
            elif mode == 'func':
                f, g = self.scale(np.linalg.norm(x)**2) / 2., None
                return f
            else:
                raise ValueError("mode incorrectly specified")
        else:
            if mode == 'both':
                f, g = self.scale(np.sum(x * self.Q_transform.linear_map(x))) / 2., self.scale(self.Q_transform.linear_map(x))
                return f, g
            elif mode == 'grad':
                f, g = None, self.scale(self.Q_transform.linear_map(x))
                return g
            elif mode == 'func':
                f, g = self.scale(np.sum(x * self.Q_transform.linear_map(x))) / 2., None
                return f
            else:
                raise ValueError("mode incorrectly specified")

    def get_conjugate(self, factor=False):

        if self.Q is None:
            as_quad = (identity_quadratic(self.coef, self.offset, 0, 0) + self.quadratic).collapsed()
            return smooth_conjugate(zero(self.shape, quadratic=as_quad))
        else:
            #XXX this needs to be tested
            sq = self.quadratic.collapsed()
            if self.offset is not None:
                sq.linear_term -= self.scale(self.Q_transform.linear_map(self.offset))
            if self.Q_transform.diagD:
                return quadratic_loss(self.shape,
                                      Q=1./(self.coef*self.Q_transform.linear_operator + sq.coef),
                                      offset=offset,
                                      quadratic=outq, 
                                      coef=1.,
                                      Qdiag=True)
            elif factor:
                return quadratic_loss(self.shape,
                                      Q=cholesky(self.coef * self.Q + sq.coef * np.identity(self.shape)),
                                      Qdiag=False,
                                      offset=offset,
                                      quadratic=outq,
                                      coef=1.)
            else:
                raise ValueError('factor is False, so no factorization was done')

class cholesky(object):

    '''

    Given :math:`Q > 0`, returns a linear transform
    that is multiplication by :math:`Q^{-1}` by
    first computing the Cholesky decomposition of :math:`Q`.

    Parameters
    ----------

    Q: array
       positive definite matrix 

    '''

    def __init__(self, Q, cholesky=None, banded=False):
        self.input_shape = Q.shape[0]
        self.output_shape = Q.shape[0]
        self.affine_offset = None
        self._Q = Q
        self.banded = banded
        if cholesky is None:
            if not self.banded:
                self._cholesky = cho_factor(Q)
            else:
                self._cholesky = cholesky_banded(Q)
        else:
            self._cholesky = cholesky

    def linear_map(self, x):
        if not self.banded:
            return cho_solve(self._cholesky, x)
        else:
            return cho_solve_banded(self._cholesky, x)

    def affine_map(self, x):
        return self.linear_map(x)

    def adjoint_map(self, x):
        return self.linear_map(x)

def squared_error(X, Y, coef=1):
    r"""
    Least squares with design $X$

    .. math::

       \frac{C}{2} \|X\beta-Y\|^2_2

    Parameters
    ----------

    X : affine_transform
        Design matrix

    Y : np.array

    """
    atom = quadratic_loss.affine(X, -Y, coef=coef)
    atom.atom.objective_vars['offset'] = 'Y'
    atom.atom.objective_template = r"""\frac{%(coef)s}{2}\left\|%(var)s\right\|^2_2"""
    return atom

def signal_approximator(signal, coef=1):
    """
    Least squares with design $I$

    .. math::

       \frac{C}{2} \|\beta-Y\|^2_2

    """
    atom = quadratic_loss.shift(signal, coef=coef)    
    atom.objective_vars['offset'] = 'Y'
    atom.objective_template = r"""\frac{%(coef)s}{2}\left\|%(var)s\right\|^2_2"""
    return atom





from copy import copy
import warnings

import numpy as np

from ..problems.composite import composite, nonsmooth, smooth_conjugate
from ..affine import linear_transform, identity as identity_transform, selector
from ..identity_quadratic import identity_quadratic
from ..atoms import _work_out_conjugate
from ..smooth import affine_smooth

from ..objdoctemplates import objective_doc_templater
from ..doctemplates import (doc_template_user, doc_template_provider)

from .seminorms import seminorm
from .cones import cone

from .mixed_lasso_cython import (mixed_lasso_bound_prox,
                                 mixed_lasso_lagrange_prox,
                                 mixed_lasso_epigraph,
                                 mixed_lasso_dual_bound_prox,
                                 seminorm_mixed_lasso,
                                 seminorm_mixed_lasso_dual)

@objective_doc_templater()
class group_lasso(seminorm):

    """
    The group lasso seminorm.
    """

    objective_template = r"""\sum_g w_g \|%(var)s[g]\|_2"""
    objective_vars = seminorm.objective_vars.copy()
    objective_vars['normklass'] = 'group_lasso'
    objective_vars['dualnormklass'] = 'group_lasso_dual'
    objective_vars['initargs'] = '[1, 1, 2, 2, 2]'

    tol = 1.0e-05

    def __init__(self, 
                 groups,
                 weights={},
                 offset=None,
                 lagrange=None,
                 bound=None,
                 quadratic=None,
                 initial=None):

        self.groups = np.asarray(groups).copy()
        shape = self.groups.shape
        seminorm.__init__(self, shape, offset=offset,
                          quadratic=quadratic,
                          initial=initial,
                          lagrange=lagrange,
                          bound=bound)

        self.weights = copy(weights)
        self._group_array = np.zeros(shape, np.intp)
        self._group_dict = {}
        self._group_inv_dict = {}

        sg = self._sorted_groupids = sorted(np.unique(self.groups))
        self._weight_array = np.ones(len(sg))
        
        for i, g in enumerate(sg):
            group = self.groups == g
            self._group_array[group] = i
            self._group_dict[g] = i
            self._group_inv_dict[i] = g
            self._weight_array[i] = self.weights.get(g, np.sqrt(group.sum()))
            self.weights[g] = self._weight_array[i]

        # buffers for cython computations

        self._l1_cython = np.array([], np.intp)
        self._unpenalized_cython = np.array([], np.intp)
        self._positive_part_cython = np.array([], np.intp)
        self._nonnegative_cython = np.array([], np.intp)
        self._norms_cython = np.zeros_like(self._weight_array)
        self._factors_cython = np.zeros_like(self._weight_array)
        self._projection_cython = np.zeros(self.shape)

    def set_weight(self, group, weight):
        """
        Set a group's weight after initialization.
        """
        idx = self._sorted_groupids.index(group)
        self._weight_array[idx] = weight
        self.weights[group] = weight
        
    def __eq__(self, other):
        if self.__class__ == other.__class__:
            return (self.shape == other.shape and 
                    np.all(self.groups == other.groups)
                    and (self.weights == other.weights)
                    and self.lagrange == other.lagrange)
        return False

    def __copy__(self):
        return self.__class__(copy(self.groups),
                              weights=copy(self.weights),
                              offset=copy(self.offset),
                              lagrange=self.lagrange,
                              bound=self.bound,
                              quadratic=copy(self.quadratic),
                              initial=copy(self.coefs))
    
    def __repr__(self):
        if self.lagrange is not None:
            if self.quadratic.iszero:
                return "%s(%s, lagrange=%s, weights=%s, offset=%s)" % \
                    (self.__class__.__name__,
                     repr(self.groups),
                     self.lagrange,
                     repr(self.weights),
                     repr(self.offset))
            else:
                return "%s(%s, lagrange=%s, weights=%s, offset=%s, quadratic=%s)" % \
                    (self.__class__.__name__,
                     repr(self.groups),
                     self.lagrange, 
                     repr(self.weights),
                     repr(self.offset),
                     repr(self.quadratic))

        if self.bound is not None:
            if self.quadratic.iszero:
                return "%s(%s, bound=%s, weights=%s, offset=%s)" % \
                    (self.__class__.__name__,
                     repr(self.groups),
                     self.bound,
                     repr(self.weights),
                     repr(self.offset))
            else:
                return "%s(%s, bound=%s, weights=%s, offset=%s, quadratic=%s)" % \
                    (self.__class__.__name__,
                     repr(self.groups),
                     self.bound,
                     repr(self.weights),
                     repr(self.offset),
                     repr(self.quadratic))

    @doc_template_user
    def get_conjugate(self):

        if self.quadratic.coef == 0:
            offset, outq = _work_out_conjugate(self.offset, 
                                               self.quadratic)

            cls = conjugate_seminorm_pairs[self.__class__]
            atom = cls(self.groups,
                       weights=self.weights,
                       bound=self.lagrange, 
                       lagrange=self.bound,
                       quadratic=outq,
                       offset=offset)
        else:
            atom = smooth_conjugate(self)
        self._conjugate = atom
        self._conjugate._conjugate = self
        return self._conjugate
    conjugate = property(get_conjugate)

    def terms(self, arg):
        """
        Return the args that are summed
        in computing the seminorm.

        >>> import regreg.api as rr
        >>> groups = [1,1,2,2,2]
        >>> penalty = rr.group_lasso(groups, lagrange=1.)
        >>> arg = [2,4,5,3,4]
        >>> list(penalty.terms(arg)) # doctest: +ELLIPSIS
        [6.3245..., 12.2474...]
        >>> penalty.seminorm(arg) # doctest: +ELLIPSIS
        18.5720...
        >>> np.sqrt((2**2 + 4**2)*2), np.sqrt((5**2 + 3**2 + 4**2) * 3.) # doctest: +ELLIPSIS
        (6.3245..., 12.2474...)
        >>> np.sqrt((2**2 + 4**2)*2) + np.sqrt((5**2 + 3**2 + 4**2) * 3.) # doctest: +ELLIPSIS
        18.5720...
        
        """
        arg = np.asarray(arg)
        norms = []
        for g in self._sorted_groupids:
            group = self.groups == g
            norms.append(np.linalg.norm(arg[group]) * self.weights[g])
        return np.array(norms)

    @doc_template_user
    def seminorm(self, arg, lagrange=None, check_feasibility=False):
        lagrange = seminorm.seminorm(self, arg, 
                                     lagrange=lagrange,
                                     check_feasibility=check_feasibility)
        arg = np.asarray(arg, np.float)

        # flush buffers for computations

        self._norms_cython[:] = 0

        return lagrange * seminorm_mixed_lasso(arg, 
                                               self._l1_cython,
                                               self._unpenalized_cython,
                                               self._positive_part_cython,
                                               self._nonnegative_cython,
                                               self._group_array,
                                               self._weight_array,
                                               self._norms_cython,
                                               False)

    @doc_template_user
    def lagrange_prox(self, arg,  lipschitz=1, lagrange=None):
        lagrange = seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        arg = np.asarray(arg, np.float)

        # flush buffers for computations

        self._norms_cython[:] = 0
        self._factors_cython[:] = 0
        self._projection_cython[:] = 0

        return mixed_lasso_lagrange_prox(arg, 
                                         float(lagrange),
                                         float(lipschitz),
                                         self._l1_cython,
                                         self._unpenalized_cython,
                                         self._positive_part_cython,
                                         self._nonnegative_cython,
                                         self._group_array,
                                         self._weight_array,
                                         self._norms_cython,
                                         self._factors_cython,
                                         self._projection_cython)

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        bound = seminorm.bound_prox(self, arg, bound)
        arg = np.asarray(arg, np.float)

        # flush buffers for computations

        self._norms_cython[:] = 0
        self._factors_cython[:] = 0
        self._projection_cython[:] = 0

        return mixed_lasso_bound_prox(arg, float(bound),
                                      self._l1_cython,
                                      self._unpenalized_cython,
                                      self._positive_part_cython,
                                      self._nonnegative_cython,
                                      self._group_array,
                                      self._weight_array,
                                      self._norms_cython,
                                      self._factors_cython,
                                      self._projection_cython)

    @doc_template_user
    def constraint(self, arg, bound=None):
        bound = seminorm.constraint(self, arg, bound=bound)
        inball = self.seminorm(arg, lagrange=1) <= bound
        if inball:
            return 0
        else:
            return np.inf

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_dual(self):
        return seminorm.dual(self)

@objective_doc_templater()
class group_lasso_dual(group_lasso):

    """
    The dual of the group lasso seminorm.
    """

    objective_template = r"""\max_g \|%(var)s[g]\|_2 / w_g"""
    objective_vars = seminorm.objective_vars.copy()
    objective_vars['normklass'] = 'group_lasso_dual'
    objective_vars['dualnormklass'] = 'group_lasso'

    tol = 1.0e-05

    def terms(self, arg, check_feasibility=True):
        """
        Return the args that are maximized
        in computing the seminorm.

        If `check_feasibility` then return `np.inf`
        for groups whose norms aren't 0 with a weight of 0.
        Otherwise, return 0 for these entries

        >>> import regreg.api as rr
        >>> groups = [1,1,2,2,2]
        >>> penalty = rr.group_lasso_dual(groups, lagrange=1.)
        >>> arg = [2,4,5,3,4]
        >>> list(penalty.terms(arg)) # doctest: +ELLIPSIS
        [3.1622..., 4.0824...]
        >>> np.sqrt((2**2 + 4**2)/2), np.sqrt((5**2 + 3**2 + 4**2) / 3.) # doctest: +ELLIPSIS
        (3.1622..., 4.0824...)
        >>> penalty.seminorm(arg) # doctest: +ELLIPSIS
        4.0824...

        """
        arg = np.asarray(arg)
        norms = []
        for g in self._sorted_groupids:
            group = self.groups == g
            w = self.weights[g]
            if w > 0:
                norms.append(np.linalg.norm(arg[group]) / w)
            else:
                if check_feasibility and np.linalg.norm(arg[group]) > 0:
                    norms.append(np.inf)  
                else:
                    norms.append(0)
        return np.array(norms)

    @doc_template_user
    def seminorm(self, arg, lagrange=None, check_feasibility=False):
        lagrange = seminorm.seminorm(self, arg, lagrange=lagrange,
                                     check_feasibility=check_feasibility)
        arg = np.asarray(arg, np.float)

        # flush buffers for computations

        self._norms_cython[:] = 0

        return lagrange * seminorm_mixed_lasso_dual(arg, 
                                                    self._l1_cython,
                                                    self._unpenalized_cython,
                                                    self._positive_part_cython,
                                                    self._nonnegative_cython,
                                                    self._group_array,
                                                    self._weight_array,
                                                    self._norms_cython,
                                                    False)

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        bound = seminorm.bound_prox(self, arg, bound)
        arg = np.asarray(arg, np.float)

        # flush buffers for computations

        self._norms_cython[:] = 0
        self._factors_cython[:] = 0
        self._projection_cython[:] = 0

        return mixed_lasso_dual_bound_prox(arg, float(bound),
                                           self._l1_cython,
                                           self._unpenalized_cython,
                                           self._positive_part_cython,
                                           self._nonnegative_cython,
                                           self._group_array,
                                           self._weight_array,
                                           self._norms_cython,
                                           self._factors_cython,
                                           self._projection_cython)

    @doc_template_user
    def lagrange_prox(self, arg,  lipschitz=1, lagrange=None):
        lagrange = seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        arg = np.asarray(arg, np.float)

        # flush buffers for computations

        self._norms_cython[:] = 0
        self._factors_cython[:] = 0
        self._projection_cython[:] = 0

        r = mixed_lasso_bound_prox(arg, lagrange / lipschitz,
                                   self._l1_cython,
                                   self._unpenalized_cython,
                                   self._positive_part_cython,
                                   self._nonnegative_cython,
                                   self._group_array,
                                   self._weight_array,
                                   self._norms_cython,
                                   self._factors_cython,
                                   self._projection_cython)
        return arg - r


    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_conjugate(self):
        return group_lasso.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return seminorm.dual(self)

@objective_doc_templater()
class group_lasso_cone(cone):
    
    """
    A superclass for the group LASSO and group LASSO dual epigraph cones.
    """

    seminorm_class = group_lasso

    def __init__(self, 
                 groups,
                 weights={},
                 offset=None,
                 quadratic=None,
                 initial=None):

        groups = np.asarray(groups)
        shape = groups.shape[0]+1
        cone.__init__(self, shape, offset=offset,
                      quadratic=quadratic,
                      initial=initial)
        cls = self.__class__
        self.snorm = cls.seminorm_class(groups,
                                        weights=weights,
                                        offset=offset,
                                        lagrange=1,
                                        bound=None,
                                        quadratic=None,
                                        initial=None)

    def __copy__(self):
        return self.__class__(copy(self.groups),
                              weights=copy(self.weights),
                              offset=copy(self.offset),
                              initial=copy(self.coefs),
                              quadratic=copy(self.quadratic))
    

    def __repr__(self):
        if self.quadratic.iszero:
            return "%s(%s, weights=%s, offset=%s)" % \
                (self.__class__.__name__,
                 repr(self.groups),
                 repr(self.weights),
                 repr(self.offset))
        else:
            return "%s(%s, weights=%s, offset=%s, quadratic=%s)" % \
                (self.__class__.__name__,
                 repr(self.groups),
                 repr(self.weights),
                 repr(self.offset),
                 repr(self.quadratic))

    @doc_template_user
    def get_conjugate(self):
        if self.quadratic.coef == 0:
            offset, outq = _work_out_conjugate(self.offset, 
                                               self.quadratic)
            cls = conjugate_cone_pairs[self.__class__]
            new_atom = cls(self.groups,
                           weights=copy(self.weights),
                           offset=offset,
                           quadratic=outq)
        else:
            new_atom = smooth_conjugate(self)
        self._conjugate = new_atom
        self._conjugate._conjugate = self
        return self._conjugate
    conjugate = property(get_conjugate)

    @property
    def weights(self):
        return self.snorm.weights

    @property
    def groups(self):
        return self.snorm.groups

    @doc_template_user
    def constraint(self, arg):
        incone = self.snorm.seminorm(arg[:-1], lagrange=1) <= (1 + self.tol) * arg[-1]
        if incone:
            return 0
        return np.inf


@objective_doc_templater()
class group_lasso_epigraph(group_lasso_cone):

    """
    The epigraph of the group lasso seminorm.
    """

    objective_template = (r"""I^{\infty}\left(\sum_g \|%(var)s[g]\|_2 """
                          + r"""\leq %(var)s[-1]\right)""")
    objective_vars = cone.objective_vars.copy()
    objective_vars['coneklass'] = 'group_lasso_epigraph'
    objective_vars['dualconeklass'] = 'group_lasso_epigraph_polar'
    objective_vars['initargs'] = '[1,1,2,2,2]'

    seminorm_class = group_lasso

    @doc_template_user
    def cone_prox(self, arg, lipschitz=1):
        arg = np.asarray(arg, np.float)

        # flush buffers for computations

        self.snorm._norms_cython[:] = 0
        self.snorm._factors_cython[:] = 0
        self.snorm._projection_cython[:] = 0

        return mixed_lasso_epigraph(arg,
                                    self.snorm._l1_cython,
                                    self.snorm._unpenalized_cython,
                                    self.snorm._positive_part_cython,
                                    self.snorm._nonnegative_cython,
                                    self.snorm._group_array,
                                    self.snorm._weight_array,
                                    self.snorm._norms_cython,
                                    self.snorm._factors_cython,
                                    self.snorm._projection_cython)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return cone.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return group_lasso_cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

@objective_doc_templater()
class group_lasso_epigraph_polar(group_lasso_cone):

    """
    The polar of the epigraph of the group lasso seminorm.
    """

    objective_template = (r"""I^{\infty}(\max_g \|%(var)s[g]\|_2 \leq """
                          + r"""-%(var)s[-1]\)""")
    objective_vars = cone.objective_vars.copy()
    objective_vars['coneklass'] = 'group_lasso_epigraph_polar'
    objective_vars['dualconeklass'] = 'group_lasso_epigraph'
    objective_vars['initargs'] = '[1,1,2,2,2]'

    seminorm_class = group_lasso_dual

    @doc_template_user
    def cone_prox(self, arg, lipschitz=1):
        arg = np.asarray(arg, np.float)

        # flush buffers for computations

        self.snorm._norms_cython[:] = 0
        self.snorm._factors_cython[:] = 0
        self.snorm._projection_cython[:] = 0

        return arg - mixed_lasso_epigraph(arg,
                                          self.snorm._l1_cython,
                                          self.snorm._unpenalized_cython,
                                          self.snorm._positive_part_cython,
                                          self.snorm._nonnegative_cython,
                                          self.snorm._group_array,
                                          self.snorm._weight_array,
                                          self.snorm._norms_cython,
                                          self.snorm._factors_cython,
                                          self.snorm._projection_cython)

    @doc_template_user
    def constraint(self, arg):
        incone = self.snorm.seminorm(arg[:-1], lagrange=1) <= (1 + self.tol) * (-arg[-1])
        if incone:
            return 0
        return np.inf

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return cone.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return group_lasso_cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

@objective_doc_templater()
class group_lasso_dual_epigraph(group_lasso_cone):

    """
    The group LASSO conjugate epigraph constraint.
    """

    objective_template = (r"""I^{\infty}(%(var)s: \max_g """ + 
                          r"""\|%(var)s[1:][g]\|_2 \leq %(var)s[0])""")

    objective_vars = cone.objective_vars.copy()
    objective_vars['coneklass'] = 'group_lasso_dual_epigraph'
    objective_vars['dualconeklass'] = 'group_lasso_dual_epigraph_polar'
    objective_vars['initargs'] = '[1,1,2,2,2]'

    seminorm_class = group_lasso_dual

    @doc_template_user
    def cone_prox(self, arg, lipschitz=1):
        arg = np.asarray(arg, np.float)

        # flush buffers for computations

        self.snorm._norms_cython[:] = 0
        self.snorm._factors_cython[:] = 0
        self.snorm._projection_cython[:] = 0

        return arg + mixed_lasso_epigraph(-arg,
                                           self.snorm._l1_cython,
                                           self.snorm._unpenalized_cython,
                                           self.snorm._positive_part_cython,
                                           self.snorm._nonnegative_cython,
                                           self.snorm._group_array,
                                           self.snorm._weight_array,
                                           self.snorm._norms_cython,
                                           self.snorm._factors_cython,
                                           self.snorm._projection_cython)

    @doc_template_user
    def constraint(self, arg):
        incone = self.snorm.seminorm(arg[:-1], lagrange=1) <= (1 + self.tol) * (arg[-1])
        if incone:
            return 0
        return np.inf

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return cone.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return group_lasso_cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

@objective_doc_templater()
class group_lasso_dual_epigraph_polar(group_lasso_cone):

    """
    The polar of the group LASSO dual epigraph constraint.
    """

    objective_template = (r"""I^{\infty}(%(var)s: \sum_g \|%(var)s[g]\|_2 \leq """
                          + r"""-%(var)s[-1]\}}""")

    objective_vars = cone.objective_vars.copy()
    objective_vars['coneklass'] = 'group_lasso_dual_epigraph_polar'
    objective_vars['dualconeklass'] = 'group_lasso_dual_epigraph'
    objective_vars['initargs'] = '[1,1,2,2,2]'


    seminorm_class = group_lasso

    @doc_template_user
    def cone_prox(self, arg,  lipschitz=1):
        arg = np.asarray(arg, np.float)

        # flush buffers for computations

        self.snorm._norms_cython[:] = 0
        self.snorm._factors_cython[:] = 0
        self.snorm._projection_cython[:] = 0

        return -mixed_lasso_epigraph(-arg,
                                      self.snorm._l1_cython,
                                      self.snorm._unpenalized_cython,
                                      self.snorm._positive_part_cython,
                                      self.snorm._nonnegative_cython,
                                      self.snorm._group_array,
                                      self.snorm._weight_array,
                                      self.snorm._norms_cython,
                                      self.snorm._factors_cython,
                                      self.snorm._projection_cython)

    @doc_template_user
    def constraint(self, arg):
        incone = self.snorm.seminorm(arg[:-1], lagrange=1) <= (1 + self.tol) * (-arg[-1])
        if incone:
            return 0
        return np.inf

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return cone.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return group_lasso_cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

conjugate_seminorm_pairs = {}
conjugate_seminorm_pairs[group_lasso_dual] = group_lasso
conjugate_seminorm_pairs[group_lasso] = group_lasso_dual

conjugate_cone_pairs = {}
conjugate_cone_pairs[group_lasso_epigraph] = group_lasso_epigraph_polar
conjugate_cone_pairs[group_lasso_epigraph_polar] = group_lasso_epigraph
conjugate_cone_pairs[group_lasso_dual_epigraph_polar] = group_lasso_dual_epigraph
conjugate_cone_pairs[group_lasso_dual_epigraph] = group_lasso_dual_epigraph_polar

# for paths

def strong_set(glasso, 
               lagrange_cur, 
               lagrange_new, 
               grad,
               slope_estimate=1):

    """
    Guess at active groups at 
    lagrange_new based on gradient
    at lagrange_cur.
    """

    dual = glasso.conjugate
    terms = dual.terms(grad)
    candidates = np.nonzero(terms >= ((slope_estimate + 1) * lagrange_new - 
                                      slope_estimate * lagrange_cur))[0]
    return [glasso._group_inv_dict[i] for i in candidates]

def check_KKT(glasso, 
              grad, 
              solution, 
              lagrange, 
              tol=1.e-2):

    """
    Check whether (grad, solution) satisfy
    KKT conditions at a given tolerance.

    Assumes glasso is group_lasso in lagrange form
    so that glasso.lagrange is not None
    """

    failing = False
    norm_soln = np.linalg.norm(solutionn)
    active = glasso.terms(solution) > tol * norm_soln
    inactive_groups = np.nonzero(~active)[0]
    active_groups = np.nozero(active)[0]
    for g in active_groups:
        group = glasso.groups == g
        subgrad_g = -grad[group]
        soln_g = solution[group]
        if np.linalg.norm(subgrad_g) < glasso.lagrange * glasso._weight_array[g] * (1 - tol):
            failing = True
        if np.linalg.norm(subgrad_g / np.linalg.norm(subgrad_g) - soln_g / np.linalg.norm(soln_g)) > tol:
            failing = True
    for g in inactive_groups:
        group = glasso.groups == g
        subgrad_g = -grad[group]
        if np.linalg.norm(subgrad_g) > glasso.lagrange * glasso._weight_array[g] * (1 + tol):
            failing = True
    return failing
